//=============================================================================
				Описание библиотеки RT_HW_BOARDS.h v.4.1 
1. Введение.
1.1.Назначение библиотеки. 
	Библиотека RT_HW_BOARDS.h создана для разработки пользовательских блоков 
    в системе визуального программирования FLProg для Arduino: https://flprog.ru - автор/разработчик Сергей Глушенко.
	Библиотека написана на C++ и может использоваться самостоятельно в среде Arduino. Библиотека является развитием подходов библиотеки RT_HW_Main.h, но не совместима с ней 
	Перед испытанием в среде FLProg, библиотека тестируется в среде Arduino и сопровождается примерами в формате ".ino".
1.2.Разработчики библиотеки: команда ecoins - ecoins@mail.ru, ecoins1994@gmail.com
1.3.Цели создания библиотеки:
		1.3.1. Реализация концепции переносимости пользовательских программ на разные контроллеры;
		1.3.2. Оптимизация по используемой памяти обоиз типов (RAM/ROM);
		1.3.3. Предоставлении пользователю минимального и достачного набора функций для 
		       проектирования программ без delay(). 
			   Так в примере RT_HW_BOARDS_Test_Event_Counter_v1 около десяти небольших 
			   параллельных процессов (в том числе и вывод на терминал значения счетчика),
			   синхронно выполняются на Arduino Nano (16 мГц) с основным периодом около 20mcs.
		1.3.4. Оптимизация по времени при использовании встроенных функций.	   
--------------------------------------------------------------------------------------
  I. ПАРАМЕТРЫ КОНТРОЛЛЕРОВ ОПРЕДЕЛЕННЫЕ ЧЕРЕЗ МАКРОПРЕДЕЛЕНИЯ И МАКРОСЫ
     Используются для обеспечения совместимости софта для различных контроллеров
//  AVR328, MEGA2560, SAM3X8E(DUE), ESP8266, ESP32, STM32F1
// Действия:
//  1. Идентифицируется тип контроллера по макросам 
//		__AVR_ATmega328__, __AVR_ATmega328P__, __AVR_ATmega168__, __AVR_ATmega88__,
//      __AVR_ATmega2560__, __AVR_ATmega2561__, __SAM3X8E__,
//		ESP8266, ESP32,
//		_BOARD_GENERIC_STM32F103C_H_, 
//      _BOARDS_GENERIC_STM32F103R_H_, _BOARD_GENERIC_STM32F103R_H_, _BOARD_GENERIC_STM32F103R8_H_, _BOARD_GENERIC_STM32F103RE_H_
//      _BOARDS_GENERIC_STM32F103V_H_, _BOARD_GENERIC_STM32F103VB_H_, _BOARDS_GENERIC_STM32F103Z_H_
//      _BOARD_GENERIC_F407V_H_;
//		и определяется один из следующих макросов, соответствующий типу контроллера:
//      	RT_HW_DEFINED_AVR328, RT_HW_DEFINED_MEGA2560, RT_HW_DEFINED_SAM3X8E (контроллер Arduino DUE),
//			RT_HW_DEFINED_ESP8266, RT_HW_DEFINED_ESP32,
//			RT_HW_DEFINED_STM32F103C, RT_HW_DEFINED_STM32F103R, RT_HW_DEFINED_STM32F103V,RT_HW_DEFINED_STM32F103Z
// 			RT_HW_DEFINED_STM32F407V 
//		Если тип контролера не установлен, определяется макрос RT_HW_ANON.
//   2. Идентифицируется групповая принадлежность контроллера:
//		RT_HW_DEFINED_ARDUINO, RT_HW_DEFINED_ESP, RT_HW_DEFINED STM32F1, RT_HW_DEFINED STM32F4
//---------------------------------------------------------------------------------------
//  II. Подключаемые библиотеки.
//  1. Arduino.h или WProgram.h (автоматический выбор);
//	2. Если контроллер поддерживает интерфейс i2c, подключается библиотека Wire.h;
//	3. Если контроллер поддерживает программный UART,подключается SoftwareSerial.h.
//		Без инициализации библиотека занимает только 20/2 байта. 
//	    SoftwareSerial работает на AVR, Mega2560, Leonardo и Micro, ESP8266, НЕ РАБОТАЕТ на DUE, ESP32, STM32!!!
//      Не все пины AVR,Mega 2560,Leonardo,Micro,Genuino 101 поддерживают прерывания изменения состояние на пине,
//	    поэтому для RX SoftwareSerial разрешено использовать:
//	    	AVR и Genuino 101 0-12, 14-20;
//      	MEGA2560 10,11,12,13,14,15,50,51,52,53,A8(62),A9(63),A10(64),A11(65),A12(66),A13(67),A14(68),A15(69).
//			Leonardo,Micro 8,9,10,11,14(MISO),15(SCK),16 (MOSI).
//   	На Arduino или Genuino 101 скорость приема составляет 57600 бит/с;
//	4. Для ESP32 подключается библиотека esp32-hal-ledc.h;
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  III. Состав таблиц под типы контроллеров.
//	1. Значения режимов для настройки пинов (макросы):	
		RT_HW_IO_MODE_INPUT						INPUT				//--Дискретный вход						(для всех)
		RT_HW_IO_MODE_INPUT_PULLUP				INPUT_PULLUP 		//--Дискретный вход с подтяжкой к +Vcc	(для всех)	
		RT_HW_IO_MODE_OUTPUT					OUTPUT				//--Дискретный выход					(для всех)
		RT_HW_IO_MODE_INPUT_FLOATING			INPUT_FLOATING		//--Дискретный вход - аналог INPUT		(STM32F)
		RT_HW_IO_MODE_INPUT_PULLDOWN			INPUT_PULLDOWN		//--Дискретный вход с подтяжкой к GND 	(STM32F,ESP32=INPUT_PULLDOWN)
		RT_HW_IO_MODE_INPUT_ANALOG				INPUT_ANALOG		//--Аналоговый вход 					(STM32F=INPUT_ANALOG)
		RT_HW_IO_MODE_OUTPUT_OPEN_DRAIN			OUTPUT_OPEN_DRAIN	//--Дискретный выход с открытым стоком	(STM32F,ESP32=OUTPUT_OPEN_DRAIN)	
		RT_HW_IO_MODE_OUTPUT_PWM				PWM					//--ШИМ выход							(STM32F=PWM)
		RT_HW_IO_MODE_OUTPUT_PWM_OPEN_DRAIN		PWM_OPEN_DRAIN		//--ШИМ выход с открытым коллектором 	(STM32F=PWM_OPEN_DRAIN
		Для других контроллеров, если настройки выше в них не допустимы:
		RT_HW_IO_MODE_INPUT_FLOATING			INPUT				//--Дискретный вход - аналог INPUT		
		RT_HW_IO_MODE_INPUT_PULLDOWN			INPUT				//--Дискретный вход с подтяжкой к GND 	
		RT_HW_IO_MODE_INPUT_ANALOG				INPUT				//--Аналоговый вход 					
		RT_HW_IO_MODE_OUTPUT_OPEN_DRAIN			OUTPUT				//--Дискретный выход с открытым стоком	
		RT_HW_IO_MODE_OUTPUT_PWM				OUTPUT				//--ШИМ выход	
	2. МАССИВЫ С ПАРАМЕТРАМИ (байтовые) (хранятся во flash). 
		Параметр с индексом 0 в каждом массиве равен кол-ву  элементов в массиве.
		Если в массиве с пинами значение <0, этот пин не используется
		RT_HW_PARAM_GENERAL[]={ 6,2,12,16, 8, 4, 3}; [1]   Пин контрольного светодиода. Если <0, Led отсутствует;(
													[2-4] Аппаратная разрядность АЦП,PWM,ЦАП; 
													[5-6] Допустимый дрейф нуля и показаний АЦП(в ед.АЦП)
		RT_HW_PARAM_ADD[]; Массив  дополнительные параметров, зарезервирован ддя будущих применений;
		RT_HW_PINS_ALL[];  Массив со всеми пинами контроллера;
		RT_HW_PINS_DIN[];  Массив с пинами дискретного ввода.	Если в массиве с пинами значение <0, этот пин не используется.
		RT_HW_PINS_ADC[];  Массив с пинами   аналового ввода.   Если в массиве с пинами значение <0, этот пин не используется.			
		RT_HW_PINS_TCH[];  Массив с пинами сенсорного  ввода.	Если в массиве с пинами значение <0, этот пин не используется. [***ESP32]					
		RT_HW_PINS_DOT[];  Массив с пинами дискретного вывода.	Если в массиве с пинами значение <0, этот пин не используется.
		RT_HW_PINS_PWM[];  Массив с пинами PWM (ШИМ)   вывода. 	Если в массиве с пинами значение <0, этот пин не используется.
		RT_HW_PINS_DAC[];  Массив с пинами аналового   вывода.	Если в массиве с пинами значение <0, этот пин не используется. [***DUE,ESP32,некоторые STM32F]
		RT_HW_PINS_UHS[];  Массив с пинами аппаратного UART.	Указываются в последовательности RX,TX;
		RT_HW_PINS_CAN[];  Массив с пинами аппаратного CAN.		Указываются в последовательности CANRX,CANTX;
		RT_HW_PINS_USS[];  Массив с пинами программного UART.	Указываются в последовательности CANRX,CANTX;)			
		RT_HW_PINS_I2C[];  Массив с пинами i3c.					Указываются в последовательности SDA,SCL;
		RT_HW_PINS_SPI[];  Массив с пинами SPI.					Указываются в последовательности SS,MOSI,MISO,SCK;
		RT_HW_PARAM_WFI[]; Массив c параметрами Wi-Fi.			Зарезервирован. 
		RT_HW_PARAM_BTH[]; Массив c параметрами Bluetooth.		Зарезервирован. 
		RT_HW_PARAM_ETH[]; Массив c параметрами Ethernet.		Зарезервирован. 
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
  IV. СОСТАВ БИБЛИОТЕКИ:
	4.1. Класс  RT_HW_BOARDS_COUNTER_ID для работы со счетчикои.
	4.2. Класс  RT_HW_BOARDS. В библиотеке создается объект и все параметры и функции
	     доступны пользователю через RT_HW_Boards.xxx
	4.3. Файл RT_HW_BOARDS.hpp содержит таблицы с данными контроллеров.
	4.4. Файл RT_HW_BOARDS_MAIN.cpp  содержит:
		- конструктор, который инциализирует необходимые параметры;
		- функции для работы с таблицами контроллерлов.
	4.5. Файл RT_HW_BOARDS_EVENT.cpp содержит: 
		- функции для работы со временем через millis(), micros();
		- функции для работы с событиями;
		- диспетчер (планировщик) задач;
		- функции для работы с универсальным счетчиком формата int16_t;
		- несколько сервисных функций.
   V. ДОСТУПНЫЕ ПАРАМЕТРЫ И РЕГИСТРЫ:
   
   
   
   
		  
//  ---RT_HW_Task.getPastTime (&timeStart, timeEnd); - [Расчет прошедшего времени;]
//		ПРИМЕР: uint32_t timeStart; 
//				timeStart=RT_HW_MILLIS;
//				RT_HW_Task.getPastTime(timeStart,RT_HW_MIILIS); -возвращает прошедшее время.
//
//  ---RT_HW_Task.checkPastTime(uint32_t &timeStart, uint32_t timeEnd, uint32_t ust); [Контроль прошедшего времени;]
//		ПРИМЕР: uint32_t timeStart; 
//				timeStart=RT_HW_MILLIS;
//				RT_HW_Task.getPastTime(timeStart,RT_HW_MIILIS, 100); - =1 после 100 ms;		
//
//  ---RT_HW_Task.getPeriod  (uint32_t &timeStart, uint32_t timeEnd, uint32_t period); [Генератор периодических (по времени) событий;]
//		uint32_t timeStart; - резервируется регистр. 
//		1) RT_HW_Task.getEvent(timeStart,RT_HW_MIILIS,period); -генерит периодическое событие каждые period ms;
//		ПРИМЕЧАНИЕ: Перед первым вызовом функции рекомендуется выполнить timeStart=RT_HW_MILLIS;
//      2) RT_HW_Task.getEvent(timeStart,RT_HW_MICROS,period); -генерит периодическое событие каждые period mcs;
//		ПРИМЕЧАНИЕ: Перед первым вызовом функции рекомендуется выполнить timeStart=RT_HW_MICROS;   
//
//  ===КЛАСС RT_HW_TASK_EVENT_ID ---> RT_HW_TASK_EVENT_ID id; [Функции работы с событиями]
//  ---RT_HW_Task.eventPeriod(id,EN,n)       	-возвращает событие (id.event)  каждые n внешнего события EN;  
//  ---RT_HW_Task.eventBlink (id,EN,n)       	-возвращает событие (id.event)  каждые n внешнего события EN;   
//												 ЦИКЛ: по каждому событию  id.event значение id.val изменяется на противоположное (1/0);
//	---RT_HW_Task.eventNum   (id,EN,n,m)  		-возвращает событие (id.event)  каждые n внешнего события EN;  При id.event==0, id.val всегда=0.
//												 ЦИКЛ: по событию  id.event значение id.val изменяется в диапазоне от 1 до m;
//  ---RT_HW_Task.eventPulse (id,EN,n,ON,OFF)  -возвращает событие (id.event)  каждые n внешнего события EN;   
//											     ЦИКЛ: первые ON событий id.val=1, следующие OFF событий id.val=0.
//
//  ===КЛАСС RT_HW_TASK_COUNTER_ID ---> RT_HW_TASK_COUNTER_ID id; [Универсальный счетчик]; 
//	---RT_HW_Task.setCounter(id,mode,delta,top,bottom,begin) -установка параметров счетчика: 
//		 mode   -  режим работы): 'S' пила, 'T' треугольник, 'L' увеличение до верхней или нижней границы, 'C' увеличение до максиума или миниума;
//		 delta  - шаг изменения счетчика;
//		 top    - верхняя граница; 
//		 bottom - нижняя граница;
//		 begin  - начальное значение счетчика;
//	---RT_HW_Task.getCounter(nnn) -по каждому вызову id.cnt меняется на величину delta;
//
//  ===КЛАСС RT_HW_TASK_SHEDULER_ID ---> RT_HW_TASK_SHEDULER_ID id; [Планировщик задач (диспетчер)] 
//  ---RT_HW_Task.sheduler(RT_HW_TASK_SHEDULER_ID &id) - планировщик задач.
//		После включения через 10мс генерит импульс reset (длина=один цикл loop()), 
//		взводит сигнал permit и затем генерит события со след.приоритетом: 
//			id.syst - 	с периодом ttt.periodSyst(по умолчанию = 10000mcs( 10ms);
//			id.fast - 	с периодом ttt.periodFast(по умолчанию = 25000mcs( 20ms); 
//			id.slow - 	с периодом ttt.periodSlow(по умолчанию =250000mcs(100ms); 
//			id.back - 	с периодом ttt.periodBack(по умолчанию =100000mcs(250ms); 
//			id.frdm - свободные задачи, =1 когда нет других событий.
//		ПРИМЕР: RT_HW_TASK_SHEDULER_ID(id);
//			if(id.slow) {cnt++;} - каждые 100 ms увеличивается счетчик cnt;
//
//	===КЛАСС RT_HW_TASK_DELAY_ID ---> RT_HW_TASK_DELAY_ID id; [{Задержка по времени (вместо delay(v);)].
//		RT_HW_Task.getDelay(RT_HW_MILLIS, val) - генерит однократное событие через val ms  после вызова функции;
//		RT_HW_Task.getDelay(RT_HW_MICROS, val) - генерит однократное событие через val mcs после вызова функции;
//		ПРИМЕЧАНИЕ: Если время задежки=0, генерация событие не будет.
//		ПРИМЕР:	switch(step){default:
//				 case 0: if(xxx.gerDelay(RT_HW_MIILIS,100) {step++;} break;
//				 case 1: break; }}

//	Возвращает событие, когда прошло время. 
//  ПРИМЕР: uint32_t timeWork, timeStart; uint8_t step=0;
//		switch(step){default: step=0;
//		case 0: timeWork=0; 
//		case 1: timeStart=RT_HW_MILLIS; step++;
//		case 2: if(!RT_HW_Boards.checkTime_mcs(timeWork,timeStart,1000)) break; 
//				//--Здесь выполняются требуемые после задержки 1000ms действия
//				timeStart=RT_HW_MILLIS; period=1000; step++;
//		case 3: if(!RT_HW_Boards.checkTime_mcs(timeWork,timeStart, 200)) break; 
//				//--Здесь выполняются требуемые после задержки 200ms действия
//				step=1;  break;
//+++++++++++++++++++++++++++++++
//-------------------------------------------------------------------------------------------------
//		ДИСПЕТЧЕР ЗАДАЧ 
//(после включения через 10мс генерит импульс reset (длина=один цикл loop()), 
//взводит сигнал permit и затем генерит события со след.приоритетом: syst,fast,slow,back,fdm
//  frdm (freedom) - свободная задача
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
//							СЧЕТЧИК
// 'S' - пила.        Если шаг >0, увеличение до верхней границы и сразу с возвратом к нижней границе. Если шаг <0, уменьшение до нижней границы и сразу с возратом  к верхней границе
// 'T' - треугольник. Если шаг >0, увеличение до верхней границы и затем  уменьшение к нижней границе. Если шаг <0, уменьшение до нижней границы, затем увеличение к верхней границе
// 'L' - лимит.       Если шаг >0, увеличение до верхней границы и останавливает счет.				   Если шаг <0, уменьшение до нижней границы и останавливает счет.
// 'C' - предел.      Если шаг >0, увеличение до  32767  и останавливает счет.                         Если шаг <0, уменьшение до -32768 и останавливает счет. 


//  СОСТАВ:
//   ---МАКРООПРЕДЕЛЕНИЯ для функции pinMode(pin,xxx):
//		RT_HW_IO_MODE_INPUT 				-дискретный вход					(для всех контроллеров)
//		RT_HW_IO_MODE_INPUT_PULLUP 			-дискретный вход с подтяжкой к +Vcc	(для всех контроллеров)	
//		RT_HW_IO_MODE_OUTPUT				-дискретный выход					(для всех контроллеров)		
//		RT_HW_IO_MODE_INPUT_FLOATING		-дискретный вход - аналог INPUT		(STM32F)
//		RT_HW_IO_MODE_INPUT_PULLDOWN		-дискретный вход с подтяжкой к GND 	(STM32F,ESP32=INPUT_PULLDOWN)
//		RT_HW_IO_MODE_INPUT_ANALOG			-аналоговый вход 					(STM32F=INPUT_ANALOG)
//		RT_HW_IO_MODE_OUTPUT_OPEN_DRAIN		-дискретный выход с открытым стоком	(STM32F,ESP32=OUTPUT_OPEN_DRAIN)
//		RT_HW_IO_MODE_OUTPUT_PWM			-ШИМ выход							(STM32F=PWM)
//		RT_HW_IO_MODE_OUTPUT_PWM_OPEN_DRAIN	-ШИМ выход с открытым коллектором 	(STM32F=PWM_OPEN_DRAIN
//
//  ---МАКРООПРЕДЕЛЕНИЯ для работы со временем
//		RT_HW_MILLIS  -аналог millis()
//		RT_HW_MICROS  -аналог micros()
// 
//  ---КОДЫ ПЛАТ
//	   RT_HW_CODE_ANON		=-1	 RT_HW_CODE_ARDUINO =1,    RT_HW_CODE_ESP=2,  		 RT_HW_CODE_STM32F1 =4, 	RT_HW_CODE_STM32F4 =5,
//	   RT_HW_CODE_AVR328  	=11, RT_HW_CODE_MEGA2560=21,   RT_HW_CODE_SAM3X8E =31,   RT_HW_CODE_ESP8266 =41,	RT_HW_CODE_ESP32   =51,  
//	   RT_HW_CODE_STM32F103C=61, RT_HW_CODE_STM32F103R=63, RT_HW_CODE_STM32F103V=65, RT_HW_CODE_STM32F103Z=67,  
//	   RT_HW_CODE_STM32F407V=71
//  ---МАКРООПРЕДЕЛЕНИЯ ПЛАТ И ГРУППЫ ПЛАТ
//     RT_HW_DEFINED_AVR328,     RT_HW_DEFINED_MEGA2560,   RT_HW_DEFINED_SAM3X8E,    RT_HW_DEFINED_ESP8266,    RT_HW_DEFINED_ESP32
//	   RT_HW_DEFINED_STM32F103C, RT_HW_DEFINED_STM32F103R, RT_HW_DEFINED_STM32F103V, RT_HW_DEFINED_STM32F103Z, RT_HW_DEFINED_STM32F407V
// 
//	---МАКРООПРЕДЕЛЕНИЯ КОДА ГРУППЫ ПЛАТ
//	   RT_HW_CODE_GROUP
//
//  ---МАССИВЫ С ПАРАМЕТРАМИ ПЛАТ AVR328,MEGA250, SAM3X8E(DUE), ESP8266,ESP32,STM32F1 (файл RT_HW_BOARDS_TYPE)
//=========================== МЕТОДЫ ==============================================================
//  ----Функция getIndexNum	(uint8_t index, uint8_t num,char mode='p');	
//      Возвращает значение массива по его индексу и номеру : mode 'u'-в формате uint8_t,'i' в формате int8_t, как номер пина(>=0 или -1);
//      ПРИМЕР: RT_HW_Boards.getIndexNum(RT_INDEX_PIN_ALL, 2;) - возвращает второе (начиная с 1) значение из массива пинов RT_HW_PINS_ALL;
//				RT_HW_Boards.getIndexNum(RT_INDEX_PIN_ALL, 0); - возвращает кол-во значений в массива пинов RT_HW_PINS_ALL;
//  ---checkPin	(uint8_t index, uint8_t pin) - проверяет наличие пина в массиве.  Возвращает 1, если номер указанного пина (pin) 
///                                                                                найден в массиве пинов с указанным индексом (index).
//      ПРИМЕР: RT_HW_Boards.checkPin(RT_INDEX_PIN_ADC, 2);    - возвращает 1, если пин 2 допустимо использовать как аналоговый вход; 
//
//  ---getPinLed() - возвращает номер контрольного пина (1-й элемент в массиве RT_HW_PARAM_GENERAL);	
//		ПРИМЕР: RT_HW_Boards.getPinLed();
//
//  ---Функция  getPastTime (&timeStart, timeEnd); - получение прошедшего времени:
//		ПРИМЕР: uint32_t timeStart; 
//				timeStart=RT_HW_MILLIS;
//				RT_HW_Boards.getPastTime(Start,RT_HW_MIILIS); -возвращает прошедшее время.

//  ---Функция checkPastTime (uint32_t &timeStart, uint32_t timeEnd, uint32_t ust); -возвращает событие после истечении времени=ust;
//		ПРИМЕР:	timeStart=RT_HW_MILLIS;
//				RT_HW_Boards.getPastTime(tmeStart,RT_HW_MIILIS, 500); -возвращает 1 через 500 ms;

//  ---Функция getPeriod (uint32_t & timeStart, uint32_t timeEnd, uint32_t period); -генерит периодическое событие;
//		ПРИМЕР: uint32_t timeStart=RT_HW_MIILIS; (однократно)
//				RT_HW_Boards.getPeriod(timeStart,RT_HW_MIILIS,200); -генерит периодическое событие через 200 ms;
//
//  ---Функция eventPeriod(uint8_t &work,uint8_t event,uint8_t num); возвращает 1 каждые событий event;
//		ПРИМЕР: uint8_t work=0; 
//				RT_HW_Boards.getPeriod(timeStart,RT_HW_MIILIS,200); -генерит периодическое событие через 200 ms;
//
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
// 	   ГЕНЕРАТОР НОМЕРОВ ЗАДАЧ (ОТ ВНЕШНЕГО СОБЫТИЯ)
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ---Функция eventNum (uint8_t &work,uint8_t event,uint8_t num) - генератор номеров задач (от внешнего события) -
//															на каждое событие возвращается значения от 1 до num, когда события нет вовзращается 0, 
//		ПРИМЕР: uint8_t work; (однократно)
//				RT_HW_Boards.eventNum(&work, event, 5) - возвращает номера задач от 1 до 5; 
//
//  ---Функция getBlink  (uint8_t &work);			   - возвращает поочередно 1 и 0("мерцание" битовой переменной).
//		ПРИМЕР: uint8_t work=0; (однократно)
//				RT_HW_Boards.getBlink(work);
//
//  ---Функция changeByte(uint8_t &work, uint8_t &val);	//-Возвращает событие при изменение байтовой переменной val; 												
//		ПРИМЕР: uint8_t work;
//				RT_HW_Boards.changeByte(work);	


//-------------------------------------------------------------------------------------------------