//++++++++++++++++  RT_HW_BOARDS.h ++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  ПАРАМЕТРЫ КОНТРОЛЛЕРОВ ОПРЕДЕЛЕННЫЕ 
//                 ЧЕРЕЗ МАКРОПРЕДЕЛЕНИЯ И МАКРОСЫ
//  Используются для обеспечения совместимости софта для различных контроллеров
//  AVR328, MEGA2560, SAM3X8E(DUE), ESP8266, ESP32, 
//  STM32F103C, STM32F103R, STM32F103V(не тестировался), STM32F103Z, STM32F407(в разработке)
//  и с учетом возможности использования в системе визуального программирования FLProg (www.flprog.ru)
//  Версия 4.1;	ecoins@mail.ru 14.06.2020г.
// --------------------------------------------------------------------------------------											
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#ifndef    RT_HW_BOARDS_h
  #define  RT_HW_BOARDS_h
#if defined(ARDUINO) && ARDUINO >= 100 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//     			НОМЕР ВЕРСИИ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_BOARDS_VERSION		42		//--Номер версии
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//     МАКРОСЫ ДЛЯ РАБОТЫ СО ВРЕМЕНЕМ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#ifndef RT_HW_GLOBAL
#define RT_HW_GLOBAL
 #define RT_HW_MILLIS millis()	//--
 #define RT_HW_MICROS micros()	//-- 
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//     МАКРОСЫ ДЛЯ РАБОТЫ С PROGMEM (FLASH-памятью) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//#define _RT_HW_READ_PGM(a,b) 	pgm_read_byte_far(a+b)		//--Чтение int8_t
//#if defined (__AVR_ATmega328__) || defined (__AVR_ATmega328P__) || defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__)
//#define _RT_HW_READ_PGM(a,b) 	pgm_read_byte(a+b)				//--Чтение int8_t
//#define _RT_HW_INT8_PGM(a)   	const int8_t   a[]  PROGMEM  	//--Создание в PROGMEM массива int8_t c именем а произвольной длины 
//#else
//#define _RT_HW_READ_PGM(a,b) 	pgm_read_byte_far(a+b)			//--Чтение int8_t с адресацией более 64kb
//#define _RT_HW_READ_PGM(a,b) 	pgm_read_byte(a+b)			//--Чтение int8_t с адресацией более 64kb
//#define _RT_HW_INT8_PGM(a)   	const int8_t   a[]  PROGMEM  	//--Создание в PROGMEM массива int8_t c именем а произвольной длины 	
//#endif
//#define _RT_HW_BYTE_PGM(a)   	const uint8_t  a[]  PROGMEM  //--Создание в PROGMEM массива uint8_t c именем а произвольной длины 
//#define _RT_HW_INT8_PGM_2(a,b)	const int8_t   a[b] PROGMEM  //--Создание в PROGMEM массива int8_t с именем а длиной b
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//     ЗНАЧЕНИЯ ИНДЕКСОВ РАБОЧИХ МАССИВОВ  И КОЛ-ВО МАССИВОВ 
//PARAM: GEN,LINK,CAN,WIFI,PARAM_BTH,PARAM_ETH; PIN: ALL,DIN,ADC,TCH,DOT,PWM,DAC,UHS,USS,I2C,SPI,CAN;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++		
enum  { RT_HW_INDEX_PARAM_GEN=0, RT_HW_INDEX_PARAM_LINK, RT_HW_INDEX_PARAM_CAN, RT_HW_INDEX_PARAM_WIFI, RT_HW_INDEX_PARAM_BTH,	RT_HW_INDEX_PARAM_ETH, 
		RT_HW_INDEX_PINS_ALL,    RT_HW_INDEX_PINS_DIN,   RT_HW_INDEX_PINS_ADC,  RT_HW_INDEX_PINS_TCH,	
		RT_HW_INDEX_PINS_DOT,    RT_HW_INDEX_PINS_PWM,   RT_HW_INDEX_PINS_DAC,  
		RT_HW_INDEX_PINS_UHS,    RT_HW_INDEX_PINS_USS,   RT_HW_INDEX_PINS_I2C,  RT_HW_INDEX_PINS_SPI,  RT_HW_INDEX_PINS_CAN};	
#define RT_HW_QNT_INDEX		17			//--Кол-во массивов с параметрами
enum  {RT_HW_BEGIN_PARAM=RT_HW_INDEX_PARAM_GEN, RT_HW_END_PARAM=RT_HW_INDEX_PARAM_ETH,
	   RT_HW_BEGIN_PINS =RT_HW_INDEX_PINS_ALL,  RT_HW_END_PINS=RT_HW_INDEX_PINS_DAC,
       RT_HW_BEGIN_LINK =RT_HW_INDEX_PINS_UHS,  RT_HW_END_LINK=RT_HW_INDEX_PINS_CAN};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		КОДЫ ПЛАТ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
enum { RT_HW_CODE_ANON=0, RT_HW_CODE_AVR328=11, RT_HW_CODE_MEGA2560=21, RT_HW_CODE_SAM3X8E =31,   
	   RT_HW_CODE_GENERIC_ESP8266 	=41,	  RT_HW_CODE_ESP32_DEV_MODULE  =51,  
	   RT_HW_CODE_GENERIC_STM32F103C=61,      RT_HW_CODE_GENERIC_STM32F103R=63, RT_HW_CODE_GENERIC_STM32F401 =71};	   
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		МАКРОСЫ ДЛЯ РАБОТЫ С UART-ами
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define	RT_HW_UART_UHS_SPEED	115200	//--Скорость HS UART (для всех)	
#define	RT_HW_UART_USS_SPEED    9600	//--Скорость SS UART (программный UART)
#define	RT_HW_WIRE_FREQ    		400		//--Скорость шины i2c по умолчанию
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		МАКРОСЫ ИДЕНТИФИКАЦИИ КОНТРОЛЛЕРОВ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_DEFINED_AVR328   	 defined (__AVR_ATmega328__) || defined (__AVR_ATmega328P__) || defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__) //--Arduino UNO, Nano,Pro,Mini
#define RT_HW_DEFINED_MEGA2560 	 defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__) //--Arduino Mega2560, 2561
#define RT_HW_DEFINED_SAM3X8E  	 defined(__SAM3X8E__)										//--Arduino DUE
#define RT_HW_DEFINED_ESP8266  	 defined(ESP8266)											//--ESP8266
#define RT_HW_DEFINED_ESP32    	 defined(ESP32)												//--ESP32
#define RT_HW_DEFINED_STM32F103C defined(_BOARD_GENERIC_STM32F103C_H_) 						//--STM103C
#define RT_HW_DEFINED_STM32F103R defined(_BOARDS_GENERIC_STM32F103R_H_) || defined(_BOARD_GENERIC_STM32F103R_H_) || defined(_BOARD_GENERIC_STM32F103R8_H_)|| defined(_BOARD_GENERIC_STM32F103RE_H_) 
#define RT_HW_DEFINED_STM32F103V defined(_BOARDS_GENERIC_STM32F103V_H_) || defined(_BOARD_GENERIC_STM32F103VB_H_)
#define RT_HW_DEFINED_STM32F103Z defined(_BOARDS_GENERIC_STM32F103Z_H_)
#define RT_HW_DEFINED_STM32F407V defined(_BOARD_GENERIC_F407V_H_) || defined(_BOARD_DISCOVERY_F4_H_ ) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		ИНДИВИДУАЛЬНАЯ НАСТРОЙКА КОНТРОЛЛЕРОВ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#include "RT_HW_BOARDS_71_PARAM_ARDUINO.hpp" 	//--Файл выбора контроллера типа ARDUINO
#include "RT_HW_BOARDS_72_PARAM_ESP8266.hpp"	//--Файл выбора контроллера типа ESP8266
#include "RT_HW_BOARDS_73_PARAM_ESP32.hpp"		//--Файл выбора контроллера типа ESP32
#include "RT_HW_BOARDS_74_PARAM_STM32F1.hpp"	//--Файл выбора контроллера типа STM32F1
#include "RT_HW_BOARDS_75_PARAM_STM32F4.hpp"	//--Файл выбора контроллера типа STM32F4
//=================================================================================================
//				ANON: ANON 
//=================================================================================================
#ifndef		RT_HW_BOARD_CODE
#include 	"RT_HW_BOARDS_61_GROUP_ARDUINO.hpp"
//---ОСНОВНЫЕ ПАРАМЕТРЫ----------------------------------------------------------------------------
	#define RT_HW_BOARD_CODE			RT_HW_CODE_ANON			//--Код контроллера;
	#define RT_HW_BOARD_NAME			"ANON"					//--Наименование контроллера;
	#define RT_HW_PIN_LED				-1						//--Номер пина со светодиодом;
//---УСТАНОВКА НАЛИЧИЯ ИНТЕРФЕСОВ------------------------------------------------------------------
//PARAM: GEN,LINK,CAN,WFI,PARAM_BTH,PARAM_ETH; PIN: ALL,DIN,ADC,TCH,DOT,PWM,DAC,UHS,USS,I2C,SPI,CAN;	
//---ПОДКЛЮЧЕНИЕ БИБЛИОТЕК---
//---СОЗДАНИЕ МАССИВОВ (int8_t(-127/127); array[0]=кол-во параметров в массиве, далее значение параметров;);
// PARAM: GEN,LINK,CAN,WFI,BTH,ETH; PINS: ALL,DIN,ADC,TCH,DOT,PWM,DAC,UHS,USS,I2C,SPI,CAN;
// GEN: Байты 1-3 разрядность АЦП,PWM,ЦАП; Байты 4 и 5: дрейф нуля и дрейф показаний АЦП(в ед.АЦП);
// LINK: Байт 1:UART,UART1-6,UATR_SS; Байт 2: i2c,i2c1,0,0,spi,spi1,0,0; Байт 3: CAN,WI-FI,BlueTooth,Ethernet;	
_RT_HW_INT8_PGM(RT_HW_PARAM_GEN)	={ 6,0,0,0,0,0,8}; 
_RT_HW_INT8_PGM(RT_HW_PARAM_LINK)	={ 3,0,0,0};	//--Наличие каналов связи; 	
_RT_HW_INT8_PGM(RT_HW_PARAM_CAN)	={ 0};			//--Параметры CAN;	
_RT_HW_INT8_PGM(RT_HW_PARAM_WIFI)	={ 0};			//--Параметры Wi-Fi;
_RT_HW_INT8_PGM(RT_HW_PARAM_BTH)	={ 0};			//--Параметры BlueTooth;
_RT_HW_INT8_PGM(RT_HW_PARAM_ETH)	={ 0};			//--Параметры Ethernet;
//---ПИНЫ ВВОДА/ВЫВОДА-----------------------------------------------------------------------------	
_RT_HW_INT8_PGM(RT_HW_PINS_ALL)		={ 0};
_RT_HW_INT8_PGM(RT_HW_PINS_DIN)		={ 0};
_RT_HW_INT8_PGM(RT_HW_PINS_ADC)		={ 0};	
_RT_HW_INT8_PGM(RT_HW_PINS_TCH)		={ 0};									
_RT_HW_INT8_PGM(RT_HW_PINS_DOT)		={ 0}; 
_RT_HW_INT8_PGM(RT_HW_PINS_PWM)		={ 0};
_RT_HW_INT8_PGM(RT_HW_PINS_DAC)		={ 0};	
//---ПИНЫ КАНАЛОВ СВЯЗИ----------------------------------------------------------------------------
_RT_HW_INT8_PGM(RT_HW_PINS_UHS)		={ 0};			//--RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_USS)		={ 0};			//--RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_I2C)		={ 0};   		//--SDA,SCL;
_RT_HW_INT8_PGM(RT_HW_PINS_SPI)		={ 0};			//--SS,MOSI,MISO,SCK;
_RT_HW_INT8_PGM(RT_HW_PINS_CAN)		={ 0};			//--CANRX,CANTX;	
//-------------------------------------------------------------------------------------------------
#endif
//-------------------------------------------------------------------------------------------------
//#include RT_HW_BOARDS_01_ADD.hpp			//--Определения привязанные к типу контроллера
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//				КЛАСС RT_HW_BOARDS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_BOARDS{
struct PRIMARY 	{uint16_t    uart:1,uart1:1,uart2:1,uart3:1,uartSS:1,i2c:1,i2c_1:1,spi:1,spi_1:1;}; 
struct DEPTH   	{int8_t sys,adc,adc_null,adc_drift,pwm,dac;};
struct STEP		{uint8_t dir;}; 
struct BUFF		{uint16_t v16, vm16; int16_t iv16; uint8_t v8; uint32_t v32;};
struct VARIA	{int8_t led; int8_t pSDA, pSCL;};	

public:					
//----- Параметры -----------------------------------------------------------------------
#pragma pack(push, 4)	//--Установка выравнивания до 4 байт									
PRIMARY		prim;		//--флаги инсталяции интерфейсов
DEPTH		depth;
STEP		step;
BUFF		buff;
VARIA		varia;
uint16_t	pwmChannelFlag; //--Регистр флагов занятости каналов PWM(для ESP32);
#pragma pack(pop)		//--Восстановление предыдущей настройки		
//----- Методы ---------------------------------------------------------------------------
RT_HW_BOARDS(); 	//--Конструктор
//-------------------------------------------------------------------------------------------------
//		PINS: ФУНКЦИИ ДЛЯ ДОСТУПА К ДАННЫМ МАССИВОВ ОПИСАНИЯ КОНТРОЛЛЕРОВ;			
//-------------------------------------------------------------------------------------------------
int16_t  getIndexNum(uint8_t index, uint8_t num, char mode='p');//--Возвращает значение массива: mode: 'u'->uint8_t; 'i'-> int8_t; 'p'->номер пина(>=0 или -1), 
//-------------------------------------------------------------------------------------------------
//		УТИЛИТЫ (определены в текущем файле)		
//-------------------------------------------------------------------------------------------------
uint8_t getFistBitUint16(uint16_t &reg) {for(uint8_t i=0;i<=15;i++) {if(!bitRead(reg,i)) return i;} return -1;};
uint16_t maxBitDepth(uint8_t v) {if(v>=16) return 0xFFFF; if(v==0) return 0; return ((1<<v)-1);};
uint16_t normADC  	(uint16_t v) {return map(v,0,maxBitDepth(depth.adc),0,maxBitDepth(depth.sys));}; //--Нормализация прочитанного ADC значения;
uint16_t normPWM  	(uint16_t v) {return map(v,0,maxBitDepth(depth.sys),0,maxBitDepth(depth.pwm));}; //--Нормализация выводимого   PWM значения;
uint16_t normDAC  	(uint16_t v) {return map(v,0,maxBitDepth(depth.sys),0,maxBitDepth(depth.dac));}; //--Нормализация выводимого   DAC значения;
bool	 checkPin(uint8_t index, uint8_t pin){if((pin<0) || (pin>127)) return 0;
buff.v8=getIndexNum(index,0); 
for(int8_t m=1;m<=buff.v8;m++) {if(getIndexNum(index,m)==pin) return 1;} return 0;};//--Проверяет наличия пина в массиве; 	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		           TASK: РАБОТА СО ВРЕМЕНЕМ, СОБЫТИЯМИ, ДИСПЕТЧЕР
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++																						 
bool     pinDigitalRead (uint8_t &dir, int8_t pin, char mode, char inv);			//--Чтение дискретного пина;
uint16_t pinAnalogRead  (uint8_t &dir, uint16_t &ago,uint16_t &sensor, int8_t pin); //--Чтение аналогово пина;
uint16_t pinTouchRead   (uint8_t &dir, int8_t pin); 								//--Чтение сенсорного пина (ESP32);
int16_t  pinHall();																	//--Чтение датчика Холла;
void     pinDigitalWrite(uint8_t &dir, uint8_t &ago, int8_t pin, char open, char change, bool val); //--Запись на дискретный пин;
void     pinDigitalBlink(uint8_t &dir, uint8_t &ago, int8_t pin); 					//--Мерцание на дискретном пине;
void     pinPWM 		(uint8_t &dir, uint8_t &num, uint16_t &device, int8_t pin, char open, uint16_t freq, uint16_t val); //--Запись на пин PWM;
void 	 pinDAC         (uint8_t &dir, uint16_t &device, int8_t  pin, uint16_t val);//--Запись на пина DAC;
bool     byteToBool		(uint8_t val) {if(val) return 1; return 0;};				//--Преобразование байта в Bool
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		LINK: РАБОТА С ИНТЕРФЕЙСАМИ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void	i2c_init();
void	spi_begin();
void    HC595(uint8_t &dir, uint8_t &ago, uint8_t cs, uint8_t val){
switch(dir){
	case 0: if(!prim.spi) break; dir++;
	case 1:	pinMode(cs,OUTPUT); ago=val; digitalWrite(cs, LOW); SPI.transfer(val); digitalWrite(cs, HIGH);
			dir++; break;
	case 2: if(ago!=val) {ago=val; digitalWrite(cs, LOW); SPI.transfer(val); digitalWrite(cs, HIGH);} break;
}};
//void RT_HW_BOARDS:: i2c_begin(){Wire.begin(); Wire.setClock(i2c.freq*(uint32_t)1000); }; //--Инициализация шины i2c; 
//void RT_HW_BOARDS:: i2c_begin(){Wire.begin(i2c.sda,i2c.scl,(i2c.freq*(uint32_t)1000)); Wire.setClockStretchLimit(260);};
//void RT_HW_BOARDS:: i2c_begin(){Wire.begin(i2c.sda,i2c.scl,(i2c.freq*(uint32_t)1000));};
//void RT_HW_BOARDS:: i2c_begin(){Wire.begin();};
//bool    i2c_find (uint8_t &dir, uint8_t&cnt, uint16_t &period, uint32_t &timeDelay,  int8_t adr, uint8_t cntMax=10);
//=======================================================================================																			 
}; //--конец класса
//============Глобальные ссылки на созданные объекты	
extern RT_HW_BOARDS		RT_HW_Boards;	//----Глобальная ссылка на созданный объект RT_HW_BOARDS
//=======================================================================================
#endif //#ifndef    RT_HW_BOARDS_h
//=======================================================================================

