//++++++++++++++++  RT_HW_BOARDS_PARAM_ARDUINO.h ++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  			ПАРАМЕТРЫ КОНТРОЛЛЕРОВ ГРУППЫ ARDUINO
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//				ARDUINO: AVR ATmega328,328P,168,88 
//=================================================================================================
#ifndef		RT_HW_BOARD_CODE
#if 		RT_HW_DEFINED_AVR328
#include 	"RT_HW_BOARDS_61_GROUP_ARDUINO.hpp"
//---ОСНОВНЫЕ ПАРАМЕТРЫ----------------------------------------------------------------------------
	#define RT_HW_BOARD_CODE			RT_HW_CODE_AVR328			//--Код контроллера;
	#define RT_HW_BOARD_NAME			"AVR328"					//--Наименование контроллера;
	#define RT_HW_PIN_LED				13							//--Номер пина со светодиодом;
//---УСТАНОВКА НАЛИЧИЯ ИНТЕРФЕСОВ------------------------------------------------------------------
	#define RT_HW_UART					//--Наличие UART Serial (консоль;)		 
	#define	RT_HW_UART_SS				//--Наличие программного UART;	
	#define RT_HW_I2C					//--Наличие интерфейса i2c
	#define RT_HW_SPI					//--Наличие интерфейсв SPI	
//---ПОДКЛЮЧЕНИЕ БИБЛИОТЕК-------------------------------------------------------------------------	
	#include <SoftwareSerial.h>			//--Библиотека программного UART. Без использования занимает 20/2 байта;
	#include <Wire.h>					//--Подключение библиотеки i2c;	   	
	#include <SPI.h>					//--Подключение библиотеки SPI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//     МАКРОСЫ ДЛЯ РАБОТЫ С PROGMEM (FLASH-памятью) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define _RT_HW_INT8_PGM(a)   			const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t c именем а произвольной длины 
#define _RT_HW_READ_PGM(a,b) 			pgm_read_byte(a+b)			//--Чтение int8_t	//--Чтение int8_t (для <64 kB);	
//---СОЗДАНИЕ МАССИВОВ (int8_t(-127/127); array[0]=кол-во параметров в массиве, далее значение параметров;);
// PARAM: GEN,LINK,CAN,WFI,BTH,ETH; PINS: ALL,DIN,ADC,TCH,DOT,PWM,DAC,UHS,USS,I2C,SPI,CAN;
// GEN: Байты 1-3:разрядность АЦП,PWM,ЦАП; Байты 4 и 5: дрейф нуля и дрейф показаний АЦП(в ед.АЦП); Байт 6 -частота процессора;
// LINK: Байт 1:UART,UART1-6,UATR_SS; Байт 2: i2c,i2c1,0,0,spi,spi1,0,0; Байт 3: CAN,WI-FI,BlueTooth,Ethernet;	
_RT_HW_INT8_PGM(RT_HW_PARAM_GEN)	={ 6, 10,8,0,4,3,16}; 			//--depth ADC,PWM,DAC,null,drift,Fcpu;
_RT_HW_INT8_PGM(RT_HW_PARAM_LINK)	={ 3,0x81,0x11,0};					//--1)UART,SS; 2)i2c,spi; 3)--
_RT_HW_INT8_PGM(RT_HW_PARAM_CAN)	={ 0};							//--Параметры CAN;	
_RT_HW_INT8_PGM(RT_HW_PARAM_WIFI)	={ 0};							//--Параметры Wi-Fi;
_RT_HW_INT8_PGM(RT_HW_PARAM_BTH)	={ 0};							//--Параметры BlueTooth;
_RT_HW_INT8_PGM(RT_HW_PARAM_ETH)	={ 0};							//--Параметры Ethernet;																			
//---ПИНЫ ВВОДА/ВЫВОДА-----------------------------------------------------------------------------	
_RT_HW_INT8_PGM(RT_HW_PINS_ALL)		={22,  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,21};
_RT_HW_INT8_PGM(RT_HW_PINS_DIN)		={20,  2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,21};
_RT_HW_INT8_PGM(RT_HW_PINS_ADC)		={ 8, 14,15,16,17,18,19,20,21};
_RT_HW_INT8_PGM(RT_HW_PINS_TCH)		={ 0};																			
_RT_HW_INT8_PGM(RT_HW_PINS_DOT)		={18,  2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19};
_RT_HW_INT8_PGM(RT_HW_PINS_PWM)		={ 6,  3, 5, 6, 9,10,11};
_RT_HW_INT8_PGM(RT_HW_PINS_DAC)		={ 0};
//---ПИНЫ КАНАЛОВ СВЯЗИ----------------------------------------------------------------------------
_RT_HW_INT8_PGM(RT_HW_PINS_UHS)		={ 2,  0, 1};					//--RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_USS)		={ 2,  8, 9};					//  RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_I2C)		={ 2, 18,19};					//--SDA,SCL;
_RT_HW_INT8_PGM(RT_HW_PINS_SPI)		={ 4, 10,11,12,13};				//--SS,MOSI,MISO,SCK;
_RT_HW_INT8_PGM(RT_HW_PINS_CAN)		={ 0};							//--CANRX,CANTX;
#endif
#endif
//=================================================================================================
//				ARDUINO: AVR ATmega2560,256
//=================================================================================================	
#ifndef		RT_HW_BOARD_CODE
#if			RT_HW_DEFINED_MEGA2560 
#include 	"RT_HW_BOARDS_61_GROUP_ARDUINO.hpp"
//---ОСНОВНЫЕ ПАРАМЕТРЫ----------------------------------------------------------------------------
	#define RT_HW_BOARD_CODE			RT_HW_CODE_MEGA2560			//--Код контроллера;
	#define RT_HW_BOARD_NAME			"MEGA2560"					//--Наименование контроллера;
	#define RT_HW_PIN_LED				13							//--Номер пина со светодиодом;	
//---УСТАНОВКА НАЛИЧИЯ ИНТЕРФЕСОВ------------------------------------------------------------------
	#define RT_HW_UART					//--Наличие UART Serial (консоль;)
	#define RT_HW_UART_1				//--Наличие UART Serial1; 
	#define RT_HW_UART_2				//--Наличие UART Serial2;
	#define RT_HW_UART_3				//--Наличие UART Serial3; 	
	#define	RT_HW_UART_SS				//--Программный UART;	
	#define RT_HW_I2C					//--Наличие интерфейса i2c
	#define RT_HW_SPI					//--Наличие интерфейса SPI		
//---ПОДКЛЮЧЕНИЕ БИБЛИОТЕК-------------------------------------------------------------------------	
	#include <SoftwareSerial.h>			//--Библиотека программного UART. Без использования занимает 20/2 байта;
	#include <Wire.h>					//--Подключение библиотеки i2c;	  
	#include <SPI.h>					//--Подключение библиотеки SPI	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//     МАКРОСЫ ДЛЯ РАБОТЫ С PROGMEM (FLASH-памятью) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define _RT_HW_INT8_PGM(a)   			const int8_t  a[] PROGMEM	//--Создание в PROGMEM массива int8_t c именем а произвольной длины; 
#define _RT_HW_READ_PGM(a,b) 			pgm_read_byte_far(a+b)		//--Чтение int8_t (для >64 kB);	
//#define _RT_HW_INT8_PGM(a)   	int8_t   a[]  //PROGMEM  	//--Создание в PROGMEM массива int8_t c именем а произвольной длины; 
//#define _RT_HW_READ_PGM(a,b) 	a[b]			//--Чтение int8_t (для >64 kB);	

//---СОЗДАНИЕ МАССИВОВ (int8_t(-127/127); array[0]=кол-во параметров в массиве, далее значение параметров;);
// PARAM: GEN,LINK,CAN,WFI,BTH,ETH; PINS: ALL,DIN,ADC,TCH,DOT,PWM,DAC,UHS,USS,I2C,SPI,CAN;
// GEN: Байты 1-3:разрядность АЦП,PWM,ЦАП; Байты 4 и 5:дрейф нуля и дрейф показаний АЦП(в ед.АЦП); Байт 6:частота процессора;
// LINK: Байт 1:UART,UART1-6,UART_SS; Байт 2: i2c,i2c1,0,0,spi,spi1,0,0; Байт 3: CAN,WI-FI,BlueTooth,Ethernet;	
_RT_HW_INT8_PGM(RT_HW_PARAM_GEN)	={ 6,10, 8, 0, 4, 3,16};	//--depth ADC,PWM,DAC,null,drift,Fcpu;
_RT_HW_INT8_PGM(RT_HW_PARAM_LINK)	={ 3,0x87,0x11,0};  					//--Дополнительные параметры (отсутствуют);	
_RT_HW_INT8_PGM(RT_HW_PARAM_CAN)	={ 0};						//--Параметры CAN;	
_RT_HW_INT8_PGM(RT_HW_PARAM_WIFI)	={ 0};						//--Параметры Wi-Fi;
_RT_HW_INT8_PGM(RT_HW_PARAM_BTH)	={ 0};						//--Параметры BlueTooth;
_RT_HW_INT8_PGM(RT_HW_PARAM_ETH)	={ 0};						//--Параметры Ethernet;
//---ПИНЫ ВВОДА/ВЫВОДА-----------------------------------------------------------------------------	
_RT_HW_INT8_PGM(RT_HW_PINS_ALL)		={70,   0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,\
                                           30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,\
										   60,61,62,63,64,65,66,67,68,69};
_RT_HW_INT8_PGM(RT_HW_PINS_DIN)		={52,   2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,\
                                           30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53};
_RT_HW_INT8_PGM(RT_HW_PINS_ADC)		={16,  54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69};	
_RT_HW_INT8_PGM(RT_HW_PINS_TCH)		={ 0};									
_RT_HW_INT8_PGM(RT_HW_PINS_DOT)		={48,   2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,\
                                           30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53}; 
_RT_HW_INT8_PGM(RT_HW_PINS_PWM)		={15,   2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,44,45,46};
_RT_HW_INT8_PGM(RT_HW_PINS_DAC)		={ 0};
//---ПИНЫ КАНАЛОВ СВЯЗИ----------------------------------------------------------------------------
_RT_HW_INT8_PGM(RT_HW_PINS_UHS)		={ 8,  0, 1,19,18,17,16,15,14};	//--RX,TX; RX1,TX1; RX2,TX2; RX3,TX4;
_RT_HW_INT8_PGM(RT_HW_PINS_USS)		={ 2, 68, 69};					//  RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_I2C)		={ 2, 20,21};					//--SDA,SCL;
_RT_HW_INT8_PGM(RT_HW_PINS_SPI)		={ 4, 53,51,50,52};				//--SS,MOSI,MISO,SCK;
_RT_HW_INT8_PGM(RT_HW_PINS_CAN)		={ 0};							//--CANRX,CANTX;
#endif
#endif
//=================================================================================================
//				ARDUINO: SAM32X8E DUE
//=================================================================================================		
#ifndef		RT_HW_BOARD_CODE
#if			RT_HW_DEFINED_SAM3X8E 
#include 	"RT_HW_BOARDS_61_GROUP_ARDUINO.hpp"
//---ОСНОВНЫЕ ПАРАМЕТРЫ----------------------------------------------------------------------------
	#define RT_HW_BOARD_CODE			RT_HW_CODE_SAM3X8E			//--Код контроллера;
	#define RT_HW_BOARD_NAME			"SAM3X8E"					//--Наименование контроллера;
	#define RT_HW_PIN_LED				13							//--Номер пина со светодиодом;	
//---УСТАНОВКА НАЛИЧИЯ ИНТЕРФЕСОВ------------------------------------------------------------------
	#define RT_HW_UART					//--Наличие UART Serial (консоль;)		
	#define RT_HW_UART_1				//--Наличие UART Serial1; 
	#define RT_HW_UART_2				//--Наличие UART Serial2;
	#define RT_HW_UART_3				//--Наличие UART Serial3; 
	#define RT_HW_I2C					//--Наличие интерфейса i2c
	#define RT_HW_I2C_1					//--Наличие дополнительного i2c;
	#define RT_HW_SPI					//--Наличие интерфейса SPI	
	#define RT_HW_CAN					//--Наличие CAN; 	
//---ПОДКЛЮЧЕНИЕ БИБЛИОТЕК-------------------------------------------------------------------------	
	#include <Wire.h>					//--Подключение библиотеки i2c;	 	
	#include <SPI.h>					//--Подключение библиотеки SPI	
//     МАКРОСЫ ДЛЯ РАБОТЫ С PROGMEM (FLASH-памятью) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define _RT_HW_INT8_PGM(a)   			const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t c именем а произвольной длины 
#define _RT_HW_READ_PGM(a,b) 			pgm_read_byte_far(a+b)		//--Чтение int8_t (для >64 kB);			
//---СОЗДАНИЕ МАССИВОВ (int8_t(-127/127); array[0]=кол-во параметров в массиве, далее значение параметров;);
// PARAM: GEN,LINK,CAN,WFI,BTH,ETH; PINS: ALL,DIN,ADC,TCH,DOT,PWM,DAC,UHS,USS,I2C,SPI,CAN;
// GEN: Байты 1-3:разрядность АЦП,PWM,ЦАП; Байты 4 и 5:дрейф нуля и дрейф показаний АЦП(в ед.АЦП); Байт 6:частота процессора;
// LINK: Байт 1:UART,UART1-6,UATR_SS; Байт 2:i2c,i2c1,0,0,spi,spi1,0,0; Байт 3:CAN,WI-FI,BlueTooth,Ethernet;	
_RT_HW_INT8_PGM(RT_HW_PARAM_GEN)	={ 6, 12,12,12,15,20,84};		//--depth ADC,PWM,DAC,null,drift,Fcpu;
_RT_HW_INT8_PGM(RT_HW_PARAM_LINK)	={ 3, B00001111,B00010011,0};  	//--1)UART,UART1-3; 2)i2c,i2c1,spi; 3)--;
_RT_HW_INT8_PGM(RT_HW_PARAM_CAN)	={ 0};							//--Параметры CAN;	
_RT_HW_INT8_PGM(RT_HW_PARAM_WIFI)	={ 0};							//--Параметры Wi-Fi;
_RT_HW_INT8_PGM(RT_HW_PARAM_BTH)	={ 0};							//--Параметры BlueTooth;
_RT_HW_INT8_PGM(RT_HW_PARAM_ETH)	={ 0};							//--Параметры Ethernet;
//---ПИНЫ ВВОДА/ВЫВОДА-----------------------------------------------------------------------------	
_RT_HW_INT8_PGM(RT_HW_PINS_ALL)		={70,  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,\
                                          30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,\
										  60,61,62,63,64,65,66,67,68,69};
_RT_HW_INT8_PGM(RT_HW_PINS_DIN)		={54,  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,\
                                          30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53};
_RT_HW_INT8_PGM(RT_HW_PINS_ADC)		={12, 54,55,56,57,58,59,60,61,62,63,64,65};	
_RT_HW_INT8_PGM(RT_HW_PINS_TCH)		={ 0};									
_RT_HW_INT8_PGM(RT_HW_PINS_DOT)		={52,  2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,\
										  30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53}; 										  
_RT_HW_INT8_PGM(RT_HW_PINS_PWM)		={12,  2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13};
_RT_HW_INT8_PGM(RT_HW_PINS_DAC)		={ 2, 66,67};	
//---ПИНЫ КАНАЛОВ СВЯЗИ----------------------------------------------------------------------------
_RT_HW_INT8_PGM(RT_HW_PINS_UHS)		={ 8,  0, 1,19,18,17,16,15,14};	//--RX,TX; RX1,TX1; RX2,TX2; RX3,TX3;
_RT_HW_INT8_PGM(RT_HW_PINS_USS)		={ 0};							//--;
_RT_HW_INT8_PGM(RT_HW_PINS_I2C)		={ 4, 20,21,-1,-1};				//--SDA,SCL;
_RT_HW_INT8_PGM(RT_HW_PINS_SPI)		={ 4, 53,51,50,52};				//--SS,MOSI,MISO,SCK;
_RT_HW_INT8_PGM(RT_HW_PINS_CAN)		={ 2, 68,69};					//--CANRX,CANTX;	
//-------------------------------------------------------------------------------------------------	
#endif
#endif	
//=================================================================================================


