//++++++++++++++++  RT_HW_BOARDS_PARAM_ESP.h ++++++++++++++++++++++++++++++++++++++++++++++++++++++

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  			ПАРАМЕТРЫ КОНТРОЛЛЕРОВ ГРУППЫ ESP8266
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//				ESP8266: Generic ESP8266 
//=================================================================================================
#ifndef		RT_HW_BOARD_CODE	
#if			RT_HW_DEFINED_ESP8266
#include 	"RT_HW_BOARDS_62_GROUP_ESP8266.hpp"
//---ОСНОВНЫЕ ПАРАМЕТРЫ----------------------------------------------------------------------------
	#define RT_HW_BOARD_CODE			RT_HW_CODE_GENERIC_ESP8266	//--Код контроллера;
	#define RT_HW_BOARD_NAME			"Generic ESP8266"			//--Наименование контроллера;
	#define RT_HW_PIN_LED				2							//--Номер пина со светодиодом;
//---УСТАНОВКА НАЛИЧИЯ ИНТЕРФЕСОВ------------------------------------------------------------------
	#define RT_HW_UART					//--Наличие UART Serial (консоль);		 
	#define	RT_HW_UART_SS				//--Наличие программного UART;	
	#define RT_HW_I2C					//--Наличие интерфейса i2c;
	#define RT_HW_SPI					//--Наличие интерфейсв SPI;		
	#define RT_HW_WIFI					//--Наличие встроенного Wi-Fi;
//---ПОДКЛЮЧЕНИЕ БИБЛИОТЕК-------------------------------------------------------------------------	
	#include <SoftwareSerial.h>			//--Библиотека программного UART. Без использования занимает 20/2 байта;
	#include <Wire.h>					//--Подключение библиотеки i2c;	  
	#include <SPI.h>					//--Подключение библиотеки SPI;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//     МАКРОСЫ ДЛЯ РАБОТЫ С PROGMEM (FLASH-памятью) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define _RT_HW_INT8_PGM(a)   			const int8_t  a[] PROGMEM  	//--Создание в PROGMEM массива int8_t c именем а произвольной длины; 
#define _RT_HW_READ_PGM(a,b) 			pgm_read_byte_far(a+b)		//--Чтение int8_t (для >64 kB);		
//---СОЗДАНИЕ МАССИВОВ (int8_t(-127/127); array[0]=кол-во параметров в массиве, далее значение параметров;);
// PARAM: GEN,LINK,CAN,WFI,BTH,ETH; PINS: ALL,DIN,ADC,TCH,DOT,PWM,DAC,UHS,USS,I2C,SPI,CAN;
// GEN: Байты 1-3:разрядность АЦП,PWM,ЦАП; Байты 4 и 5:дрейф нуля и дрейф показаний АЦП(в ед.АЦП); Байт 6 -частота процессора;
// LINK: Байт 1:UART,UART1-6,UATR_SS; Байт 2: i2c,i2c1,0,0,spi,spi1,0,0; Байт 3: CAN,WI-FI,BlueTooth,Ethernet;
_RT_HW_INT8_PGM(RT_HW_PARAM_GEN)	={ 6, 10,10, 0,15,20,160};	//--depth ADC,PWM,DAC,null,drift,Fcpu;
_RT_HW_INT8_PGM(RT_HW_PARAM_LINK)	={ 0};  					//--Дополнительные параметры (отсутствуют).	
_RT_HW_INT8_PGM(RT_HW_PARAM_CAN)	={ 0};						//--Параметры CAN;	
_RT_HW_INT8_PGM(RT_HW_PARAM_WIFI)	={ 2, 0,1};					//--Параметры Wi-Fi;
_RT_HW_INT8_PGM(RT_HW_PARAM_BTH)	={ 0};						//--Параметры BlueTooth;
_RT_HW_INT8_PGM(RT_HW_PARAM_ETH)	={ 0};						//--Параметры Ethernet;
//---ПИНЫ ВВОДА/ВЫВОДА-----------------------------------------------------------------------------	
_RT_HW_INT8_PGM(RT_HW_PINS_ALL)		={12,  16, 5, 4, 0, 2,14,12,13,15, 3, 1,17};
_RT_HW_INT8_PGM(RT_HW_PINS_DIN)		={10,      5, 4, 0, 2,14,12,13,15, 3, 1};
_RT_HW_INT8_PGM(RT_HW_PINS_ADC)		={1,   17};	
_RT_HW_INT8_PGM(RT_HW_PINS_TCH)		={0};									
_RT_HW_INT8_PGM(RT_HW_PINS_DOT)		={11, 16, 5, 4, 0, 2,14,12,13,15,3,1}; 
_RT_HW_INT8_PGM(RT_HW_PINS_PWM)		={10,     5, 4, 0, 2,14,12,13,15,3,1};
_RT_HW_INT8_PGM(RT_HW_PINS_DAC)		={0};
//---ПИНЫ КАНАЛОВ СВЯЗИ----------------------------------------------------------------------------
_RT_HW_INT8_PGM(RT_HW_PINS_UHS)		={ 2,  3, 1};			//--RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_USS)		={ 2,13,15};			//--RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_I2C)		={ 2, 4, 5};			//--SDA,SCL;
_RT_HW_INT8_PGM(RT_HW_PINS_SPI)		={ 4, 15,13,12,14};		//--SS,MOSI,MISO,SCK;
_RT_HW_INT8_PGM(RT_HW_PINS_CAN)		={ 0};					//--CANRX,CANTX;	
//-------------------------------------------------------------------------------------------------	
#endif
#endif	
//=================================================================================================


