//++++++++++++++++  RT_HW_BOARDS_PARAM_ESP32.h ++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  			ПАРАМЕТРЫ КОНТРОЛЛЕРОВ ГРУППЫ ESP32
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//				ESP32: ESP32 DEVKIT V1
//=================================================================================================
#ifndef		RT_HW_BOARD_CODE
#if 		RT_HW_DEFINED_ESP32
#include 	"RT_HW_BOARDS_63_GROUP_ESP32.hpp"
//---ОСНОВНЫЕ ПАРАМЕТРЫ----------------------------------------------------------------------------
	#define RT_HW_BOARD_CODE			RT_HW_CODE_AVR328			//--Код контроллера;
	#define RT_HW_BOARD_NAME			"ESP32 Dev Module"			//--Наименование контроллера;
	#define RT_HW_PIN_LED				2							//--Номер пина со светодиодом;
//---УСТАНОВКА НАЛИЧИЯ ИНТЕРФЕСОВ------------------------------------------------------------------
	#define RT_HW_UART					//--Наличие UART Serial (консоль;)		 
	#define RT_HW_UART_2				//--Наличие UART Serial2;
	#define RT_HW_I2C					//--Наличие интерфейса i2c
	#define RT_HW_SPI					//--Наличие интерфейсв SPI
	#define RT_HW_WIFI					//--Наличие встроенного Wi-Fi
	#define RT_HW_BT					//--Наличие встроенного BlueTooth
//---ПОДКЛЮЧЕНИЕ БИБЛИОТЕК-------------------------------------------------------------------------	
	#include <Wire.h>					//--Подключение библиотеки i2c;	 
	#include "esp32-hal-ledc.h"			//--Подключение библиотеки ШИМ 	
	#include <SPI.h>					//--Подключение библиотеки SPI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//     МАКРОСЫ ДЛЯ РАБОТЫ С PROGMEM (FLASH-памятью) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define _RT_HW_INT8_PGM(a)   			const int8_t  a[] PROGMEM 	//--Создание в PROGMEM массива int8_t c именем а произвольной длины; 
#define _RT_HW_READ_PGM(a,b) 			pgm_read_byte_far(a+b)		//--Чтение int8_t (для >64 kB);		
//---СОЗДАНИЕ МАССИВОВ (int8_t(-127/127); array[0]=кол-во параметров в массиве, далее значение параметров;);
// PARAM: GEN,LINK,CAN,WFI,BTH,ETH; PINS: ALL,DIN,ADC,TCH,DOT,PWM,DAC,UHS,USS,I2C,SPI,CAN;
// GEN: Байты 1-3:разрядность АЦП,PWM,ЦАП; Байты 4 и 5:дрейф нуля и дрейф показаний АЦП(в ед.АЦП);
// LINK: Байт 1:UART,UART1-6,UATR_SS; Байт 2:i2c,i2c1,0,0,spi,spi1,0,0; Байт 3:CAN,WI-FI,BlueTooth,Ethernet;	
_RT_HW_INT8_PGM(RT_HW_PARAM_GEN)	={ 6, 12,12,8,5,10,240}; 			//--depth ADC,PWM,DAC,null,drift,Fcpu;
_RT_HW_INT8_PGM(RT_HW_PARAM_LINK)	={ 3,B00000101,B00000101,B00000110};//--1)UART,UART2; 2)i2c,spi; 3)Wi-Fi,BlueTooth
_RT_HW_INT8_PGM(RT_HW_PARAM_CAN)	={ 0};							//--Параметры CAN;	
_RT_HW_INT8_PGM(RT_HW_PARAM_WIFI)	={ 4, 192,168,0,1};					//--Параметры Wi-Fi;
_RT_HW_INT8_PGM(RT_HW_PARAM_BTH)	={ 6, 'B','T','h','0','0','1'}; 	//--Параметры BlueTooth -имя по умолчанию;
_RT_HW_INT8_PGM(RT_HW_PARAM_ETH)	={ 0};								//--Параметры Ethernet;																			
//---ПИНЫ ВВОДА/ВЫВОДА-----------------------------------------------------------------------------	
_RT_HW_INT8_PGM(RT_HW_PINS_ALL)		={26,  0, 1, 2, 3, 4, 5,12,13,14,15,16,17,18,19,21,22,23,25,26,27,32,33,34,35,36,39};
_RT_HW_INT8_PGM(RT_HW_PINS_DIN)		={26,  0, 1, 2, 3, 4, 5,12,13,14,15,16,17,18,19,21,22,23,25,26,27,32,33,34,35,36,39};
_RT_HW_INT8_PGM(RT_HW_PINS_ADC)		={16, 36,39,32,33,34,35, 4, 0, 2,15,13,12,14,27,25,26};	
_RT_HW_INT8_PGM(RT_HW_PINS_TCH)		={10,  4, 0, 2,15,13,12,14,27,33,32};									
_RT_HW_INT8_PGM(RT_HW_PINS_DOT)		={22,  0, 1, 2, 3, 4, 5,12,13,14,15,16,17,18,19,21,22,23,25,26,27,32,33}; 
_RT_HW_INT8_PGM(RT_HW_PINS_PWM)		={16,  2, 4, 5,13,14,15,16,17,18,19,23,25,26,27,32,33};
_RT_HW_INT8_PGM(RT_HW_PINS_DAC)		={2,  25,26};
//---ПИНЫ КАНАЛОВ СВЯЗИ----------------------------------------------------------------------------
_RT_HW_INT8_PGM(RT_HW_PINS_UHS)		={ 6,  3,1,-1,-1,16,17};	//--RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_USS)		={ 0};						//--RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_I2C)		={ 2, 21, 22};				//--SDA,SCL;
_RT_HW_INT8_PGM(RT_HW_PINS_SPI)		={ 4,  5, 23,19,18};		//--SS,MOSI,MISO,SCK;
_RT_HW_INT8_PGM(RT_HW_PINS_CAN)		={ 0};						//--CANRX,CANTX;
//-------------------------------------------------------------------------------------------------	
#endif
#endif	
//=================================================================================================


