//++++++++++++++++  RT_HW_BOARDS_PARAM_STM32F1.h ++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  			ПАРАМЕТРЫ КОНТРОЛЛЕРОВ ГРУППЫ STM32F1
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=================================================================================================
//				STM32F1: Generic STM32F103C
//=================================================================================================
#ifndef		RT_HW_BOARD_CODE
#if 		RT_HW_DEFINED_STM32F103C
#include 	"RT_HW_BOARDS_64_GROUP_STM32F1.hpp"
//---ОСНОВНЫЕ ПАРАМЕТРЫ----------------------------------------------------------------------------
	#define RT_HW_BOARD_CODE			RT_HW_CODE_STM32F103C		//--Код контроллера;
	#define RT_HW_BOARD_NAME			"Generic STM32F103C"		//--Наименование контроллера;
	#define RT_HW_PIN_LED				PC13						//--Номер пина со светодиодом;
//---УСТАНОВКА НАЛИЧИЯ ИНТЕРФЕСОВ------------------------------------------------------------------
	#define RT_HW_UART_1				//--Наличие UART Serial1; 
	#define RT_HW_UART_2				//--Наличие UART Serial2;
	#define RT_HW_UART_3				//--Наличие UART Serial3; 	
	#define	RT_HW_CAN					//--Наличие CAN; 
	#define RT_HW_I2C					//--Наличие i2c;
	#define RT_HW_I2C_1					//--Наличие дополнительного i2c;
	#define RT_HW_SPI					//--Наличие о SPI;
	#define RT_HW_SPI_1					//--Наличие дополнительного SPI	
//---ПОДКЛЮЧЕНИЕ БИБЛИОТЕК-------------------------------------------------------------------------	
	#include <Wire.h>						//--Подключение библиотеки i2c;	   	
	#include <SPI.h>					//--Подключение библиотеки SPI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//     МАКРОСЫ ДЛЯ РАБОТЫ С PROGMEM (FLASH-памятью) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define _RT_HW_INT8_PGM(a)   			const int8_t  a[] PROGMEM	//--Создание в PROGMEM массива int8_t c именем а произвольной длины; 
#define _RT_HW_READ_PGM(a,b) 			pgm_read_byte_far(a+b)		//--Чтение int8_t (для >64 kB);	
//---СОЗДАНИЕ МАССИВОВ (int8_t(-127/127); array[0]=кол-во параметров в массиве, далее значение параметров;);
// PARAM: GEN,LINK,CAN,WFI,BTH,ETH; PINS: ALL,DIN,ADC,TCH,DOT,PWM,DAC,UHS,USS,I2C,SPI,CAN;
// GEN: Байты 1-3 разрядность АЦП,PWM,ЦАП; Байты 4 и 5: дрейф нуля и дрейф показаний АЦП(в ед.АЦП);
// LINK: Байт 1:UART,UART1-6,UATR_SS; Байт 2:i2c,i2c1,0,0,spi,spi1,0,0; Байт 3:CAN,WI-FI,BlueTooth,Ethernet;	
_RT_HW_INT8_PGM(RT_HW_PARAM_GEN)	={ 6, 12,16,12,20,25,72}; 			//--depth ADC,PWM,DAC,null,drift,Fcpu;
_RT_HW_INT8_PGM(RT_HW_PARAM_LINK)	={ 3,B00001111,B00001111,B00000001};//--1)UART,SS; 2)i2c,spi; 3)CAN
_RT_HW_INT8_PGM(RT_HW_PARAM_CAN)	={ 0};								//--Параметры CAN;	
_RT_HW_INT8_PGM(RT_HW_PARAM_WIFI)	={ 0};								//--Параметры Wi-Fi;
_RT_HW_INT8_PGM(RT_HW_PARAM_BTH)	={ 0};								//--Параметры BlueTooth;
_RT_HW_INT8_PGM(RT_HW_PARAM_ETH)	={ 0};								//--Параметры Ethernet;																			
//---ПИНЫ ВВОДА/ВЫВОДА-----------------------------------------------------------------------------	
_RT_HW_INT8_PGM(RT_HW_PINS_ALL)		={35, PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7, PA8, PA9, PA10, PA11, PA12, PA13, PA14, PA15, \
									      PB0, PB1, PB2, PB3, PB4, PB5, PB6, PB7, PB8, PB9, PB10, PB11, PB12, PB13, PB14, PB15, \
                                          PC13, PC14, PC15 };
_RT_HW_INT8_PGM(RT_HW_PINS_DIN)		={30, PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7, PA8, PA9, PA10,             PA13, PA14, PA15, \
									      PB0, PB1, PB2, PB3, PB4, PB5, PB6, PB7, PB8, PB9, PB10, PB11, PB12, PB13, PB14, PB15};
_RT_HW_INT8_PGM(RT_HW_PINS_ADC)		={10, PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7, PB1, PB0};  
_RT_HW_INT8_PGM(RT_HW_PINS_TCH)		={0};									
_RT_HW_INT8_PGM(RT_HW_PINS_DOT)		={33, PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7, PA8, PA9, PA10, PA11, PA12, PA13, PA14, PA15, \
									      PB0, PB1, PB2, PB3, PB4, PB5, PB6, PB7, PB8, PB9, PB10, PB11, PB12, PB13, PB14, PB15, \
                                          PC13};
_RT_HW_INT8_PGM(RT_HW_PINS_PWM)		={12, PB0, PA7, PA6, PA3, PA2, PA1, PA0, PB6, PB7, PA10, PA9, PA8}; 
_RT_HW_INT8_PGM(RT_HW_PINS_DAC)		={0};
//---ПИНЫ КАНАЛОВ СВЯЗИ----------------------------------------------------------------------------
_RT_HW_INT8_PGM(RT_HW_PINS_UHS)		={ 8, PA12,PA11,PA10,PA9,PA3,PA2,PB11,PB10};	//--RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_USS)		={ 0};											//--RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_I2C)		={ 4, PB7,PB6,PB11,PB10};   					//--SDA,SCL;
_RT_HW_INT8_PGM(RT_HW_PINS_SPI)		={ 8, PA4,PA7,PA6,PA4, PB12,PB15,PB14,PB13};	//--SS,MOSI,MISO,SCK;
_RT_HW_INT8_PGM(RT_HW_PINS_CAN)		={ 2, PB8,PB9};									//--CANRX,CANTX;	
//-------------------------------------------------------------------------------------------------	
#endif
#endif	
//=================================================================================================
//				STM32F1: Generic STM32F103R
//=================================================================================================
#ifndef		RT_HW_BOARD_CODE
#if 		RT_HW_DEFINED_STM32F103R
#include 	"RT_HW_BOARDS_64_GROUP_STM32F1.hpp"
//---ОСНОВНЫЕ ПАРАМЕТРЫ----------------------------------------------------------------------------
	#define RT_HW_BOARD_CODE			RT_HW_CODE_STM32F103R		//--Код контроллера;
	#define RT_HW_BOARD_NAME			"Generic STM32F103R"		//--Наименование контроллера;
	#define RT_HW_PIN_LED				PB0							//--Номер пина со светодиодом;
//---УСТАНОВКА НАЛИЧИЯ ИНТЕРФЕСОВ------------------------------------------------------------------
	#define RT_HW_UART_1					//--Наличие UART Serial1; 
	#define RT_HW_UART_2					//--Наличие UART Serial2;
	#define RT_HW_UART_3					//--Наличие UART Serial3; 
	#define RT_HW_UART_4					//--Наличие UART Serial4;
	#define RT_HW_UART_5					//--Наличие UART Serial5;	
	#define	RT_HW_CAN						//--Наличие CAN;
	#define RT_HW_I2C						//--Наличие i2c;
	#define RT_HW_I2C_1						//--Наличие дополнительного i2c;
	#define RT_HW_SPI						//--Наличие о SPI;
	#define RT_HW_SPI_1						//--Наличие дополнительного SPI		
//---ПОДКЛЮЧЕНИЕ БИБЛИОТЕК-------------------------------------------------------------------------	
	#include <Wire.h>						//--Подключение библиотеки i2c;	   	
	#include <SPI.h>						//--Подключение библиотеки SPI
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//     МАКРОСЫ ДЛЯ РАБОТЫ С PROGMEM (FLASH-памятью) 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define _RT_HW_INT8_PGM(a)   			const int8_t  a[] PROGMEM	//--Создание в PROGMEM массива int8_t c именем а произвольной длины; 
#define _RT_HW_READ_PGM(a,b) 			pgm_read_byte_far(a+b)		//--Чтение int8_t (для >64 kB);		
//---СОЗДАНИЕ МАССИВОВ (int8_t(-127/127); array[0]=кол-во параметров в массиве, далее значение параметров;);
// PARAM: GEN,LINK,CAN,WFI,BTH,ETH; PINS: ALL,DIN,ADC,TCH,DOT,PWM,DAC,UHS,USS,I2C,SPI,CAN;
// GEN: Байты 1-3 разрядность АЦП,PWM,ЦАП; Байты 4 и 5: дрейф нуля и дрейф показаний АЦП(в ед.АЦП);
// LINK: Байт 1:UART,UART1-6,UATR_SS; Байт 2:i2c,i2c1,0,0,spi,spi1,0,0; Байт 3:CAN,WI-FI,BlueTooth,Ethernet;	
_RT_HW_INT8_PGM(RT_HW_PARAM_GEN)	={ 6, 12,16,12,20,25,72}; 			//--depth ADC,PWM,DAC,null,drift,Fcpu;
_RT_HW_INT8_PGM(RT_HW_PARAM_LINK)	={ 3,B00001111,B00001111,B00000001};//--1)UART,SS; 2)i2c,spi; 3)CAN
_RT_HW_INT8_PGM(RT_HW_PARAM_CAN)	={ 0};								//--Параметры CAN;	
_RT_HW_INT8_PGM(RT_HW_PARAM_WIFI)	={ 0};								//--Параметры Wi-Fi;
_RT_HW_INT8_PGM(RT_HW_PARAM_BTH)	={ 0};								//--Параметры BlueTooth;
_RT_HW_INT8_PGM(RT_HW_PARAM_ETH)	={ 0};								//--Параметры Ethernet;																			
//---ПИНЫ ВВОДА/ВЫВОДА-----------------------------------------------------------------------------	
_RT_HW_INT8_PGM(RT_HW_PINS_ALL)		={51, PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7, PA8, PA9, PA10, PA11, PA12, PA13, PA14, PA15, \
                                          PB0, PB1, PB2, PB3, PB4, PB5, PB6, PB7, PB8, PB9, PB10, PB11, PB12, PB13, PB14, PB15, \
                                          PC0, PC1, PC2, PC3, PC4, PC5, PC6, PC7, PC8, PC9, PC10, PC11, PC12, PC13, PC14, PC15, PD2};                                                                        
_RT_HW_INT8_PGM(RT_HW_PINS_DIN)		={51, PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7, PA8, PA9, PA10, PA11, PA12, PA13, PA14, PA15, \
                                          PB0, PB1, PB2, PB3, PB4, PB5,-PB6,-PB7, PB8, PB9, PB10, PB11, PB12, PB13, PB14, PB15, \
                                          PC0, PC1, PC2, PC3, PC4, PC5, PC6, PC7, PC8, PC9, PC10, PC11, PC12, PC13, PC14, PC15, PD2}; 											
_RT_HW_INT8_PGM(RT_HW_PINS_ADC)		={16, PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7, PB0, PB1, PC0, PC1, PC2, PC3, PC4, PC5};              
_RT_HW_INT8_PGM(RT_HW_PINS_TCH)		={0};									
_RT_HW_INT8_PGM(RT_HW_PINS_DOT)		={51, PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7, PA8, PA9, PA10, PA11, PA12, PA13, PA14, PA15, \
                                          PB0, PB1, PB2, PB3, PB4, PB5,-PB6,-PB7, PB8, PB9, PB10, PB11, PB12, PB13, PB14, PB15, \
										  PC0, PC1, PC2, PC3, PC4, PC5, PC6, PC7, PC8, PC9, PC10, PC11, PC12, PC13, PC14, PC15, PD2};                                        
_RT_HW_INT8_PGM(RT_HW_PINS_PWM)		={19, PA0, PA1, PA2, PA3, PA6, PA7, PA8, PA9, PA10,PB0, PB1,  PB6,  PB7,  PB8,  PB9,  PC6, PC7, PC8, PC9};
_RT_HW_INT8_PGM(RT_HW_PINS_DAC)		={2,  PA4,PA5};
//-------МАСССИВЫ С ПИНАМИ КАНАЛОВ СВЯЗИ-----------------------------------------------------------
_RT_HW_INT8_PGM(RT_HW_PINS_UHS)		={12, PA12,PA11,PA10,PA9,PA3,PA2,PB11,PB10,PC11,PC10,PD2,PC12};	//--RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_USS)		={ 0};															//--RX,TX;
_RT_HW_INT8_PGM(RT_HW_PINS_I2C)		={ 4, PB7,PB6,PB11,PB10};   									//--SDA,SCL;
_RT_HW_INT8_PGM(RT_HW_PINS_SPI)		={ 8, PA4,PA7,PA6,PA4, PB12,PB15,PB14,PB13};					//--SS,MOSI,MISO,SCK;
_RT_HW_INT8_PGM(RT_HW_PINS_CAN)		={ 2, PB8,PB9};													//--CANRX,CANTX;
//-------------------------------------------------------------------------------------------------	
#endif
#endif	
//===========================================================================================================


