//++++++++++++++++  RT_HW_TASK.h ++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  ПАРАМЕТРЫ КОНТРОЛЛЕРОВ ОПРЕДЕЛЕННЫЕ 
//                 ЧЕРЕЗ МАКРОПРЕДЕЛЕНИЯ И МАКРОСЫ
//  Используются для обеспечения совместимости софта для различных контроллеров
//  AVR328, MEGA2560, SAM3X8E(DUE), ESP8266, ESP32, 
//  STM32F103C, STM32F103R, STM32F103V(не тестировался), STM32F103Z, STM32F407(в разработке)
//  и с учетом возможности использования в системе визуального программирования FLProg (www.flprog.ru)
//  Версия 4.1;	ecoins@mail.ru 14.06.2020г.
// --------------------------------------------------------------------------------------											
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#ifndef    RT_HW_TASK_h
  #define  RT_HW_TASK_h
#if defined(ARDUINO) && ARDUINO >= 100 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
//-------------------------------------------------------------------
#define RT_HW_TASK_VERSION		12		//--Номер версии библиотеки RT_HW_CONFIG.h	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//     МАКРОСЫ ДЛЯ РАБОТЫ СО ВРЕМЕНЕМ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define RT_HW_TASK_MILLIS millis()
#define RT_HW_TASK_MICROS micros()
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//				КЛАСС RT_HW_TASK
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_TASK{
struct SHEDULER { 
struct Param {uint8_t num,cnt,qnt;};
Param    quick, fast, slow, back, frdm; 
uint8_t  dir;
uint8_t	 startSlow,startBack;
uint8_t  mean10:1,mean25:1,mean250:1;
uint8_t  cntSlow,cntBack,cnt250,qntSlow,qntBack;
uint8_t  div10,div25,div250,work; 
uint32_t timeStart10,timeStart50;
}; 
public:					
//----- Параметры -----------------------------------------------------------------------
#pragma pack(push, 4)	//--Установка выравнивания до 4 байт
SHEDULER 	Shed;							//--Объект с регистрами для диспетчера задач;
#pragma pack(pop)		//--Восстановление предыдущей настройки		
//----- Методы ---------------------------------------------------------------------------
RT_HW_TASK(); 	//--Конструктор
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		           TASK: РАБОТА СО ВРЕМЕНЕМ, СОБЫТИЯМИ, ДИСПЕТЧЕР
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++																						 
//uint32_t getTime  	(uint32_t &timeStart, uint32_t timeEnd);						//--Возвращает разницу между двумя временными точками; 
//uint8_t	 checkTime	(uint32_t &timeStart, uint32_t timeEnd, uint32_t val);			//--Проверка прошедшего времени;	
//uint8_t  checkDelay (uint32_t &timeWork,  uint32_t &timeStart, uint32_t timeEnd, uint32_t val); //--Замена delay() без торможения процессора;
uint8_t  period		(uint32_t &timeStart, uint32_t timeEnd, uint32_t period);		//--Генератор периодических событий;
void 	 sheduler(); 																//--Диспетчер (планировщик задач);
bool     divider_EN     (uint8_t &cnt, int8_t &ago, uint8_t qnt, bool EN);			//--Делитель входного события EN;
bool	 trigger_Update	(uint8_t &cnt, int8_t &ago, uint8_t qnt, bool En);			//--Триггер изменения входа с делителем;
bool	 trigger_Front	(uint8_t &cnt, int8_t &ago, uint8_t qnt, bool En);			//--Триггер фронта на входе с делителем;
bool     pulse_EN		(uint8_t &cnt, int8_t &ago, uint8_t ON, uint8_t OFF, bool EN);//--Генератор импульсов от события EN;
bool     pulse_En       (uint8_t &cnt, int8_t &ago, uint8_t ON, uint8_t OFF, bool En);//--Генератор импульсов от изменения входа En;
uint8_t	 dc8			(uint8_t  &work, uint8_t v);								//-- 8-разрядный дешифратор;
uint16_t dc16			(uint16_t &work, uint8_t v);								//--16-разрядный дешифратор; 
int16_t  Counter(int32_t &val, uint8_t &direct, uint8_t begin, uint8_t bottom, uint8_t top, char mode, uint8_t delta, bool EN, bool RST); //--Универсальный счетчик;		 

uint8_t  genMeander     (uint8_t &ago, uint8_t &cnt, uint8_t &ust);																			 
}; //--конец класса
//============Глобальные ссылки на созданные объекты	
extern RT_HW_TASK		RT_HW_Task;	//----Глобальная ссылка на созданный объект RT_HW_BOARDS
//=======================================================================================
#endif //#ifndef    RT_HW_TASK_h
//=======================================================================================

