//++++++++++++++++  RT_HW_CONSOLE.h ++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  ПАРАМЕТРЫ КОНТРОЛЛЕРОВ ОПРЕДЕЛЕННЫЕ 
//                 ЧЕРЕЗ МАКРОПРЕДЕЛЕНИЯ И МАКРОСЫ
//  Используются для обеспечения совместимости софта для различных контроллеров
//  AVR328, MEGA2560, SAM3X8E(DUE), ESP8266, ESP32, 
//  STM32F103C, STM32F103R, STM32F103V(не тестировался), STM32F103Z, STM32F407(в разработке)
//  и с учетом возможности использования в системе визуального программирования FLProg (www.flprog.ru)
//  Версия 4.1;	ecoins@mail.ru 14.06.2020г.
// --------------------------------------------------------------------------------------											
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#ifndef    RT_HW_CONSOLE_h
  #define  RT_HW_CONSOLE_h
#if defined(ARDUINO) && ARDUINO >= 100 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
//-------------------------------------------------------------------------------------------------
#define RT_HW_CONSOLE_VERSION		21			//--Номер версии библиотеки RT_HW_CONSOLE.h	  
#define RT_HW_CONSOLE_MILLIS  		millis()	//--Получение значение миллисекунд;
#define RT_HW_CONSOLE_MICROS  		micros()	//--Получение значение миллисекунд;
#define RT_HW_CONSOLE_SIZE_ARR_CHAR 10			//--Размер массива под преобразования чисел с массив char
#define RT_HW_CONSOLE_BREAK_CHAR	'~'			//--Символ "Запрещающего" символа;
#define RT_HW_CONSOLE_BLANK	        's'			//--Символ указатель, что заполнитель "пробел";
#define RT_HW_CONSOLE_PERIOD_TICK   50			//--Период системного времени в ms;
#define RT_HW_CONSOLE_UPPER  		0			//--=0 -нижний регистр, =1 - верхгий регистр;
//-------------------------------------------------------------------------------------------------
struct RT_HW_CONSOLE_VAL_ID   {uint32_t val;  uint16_t len:4, index:4, num:4, mess:4;  uint8_t mode:2, minus:1, point:2, pnt:1;}; //--Структура для выводимой переменной
struct RT_HW_CONSOLE_EVENT_ID {uint8_t  blink:1, period:1, fresh:1, dir:3;     uint8_t eventCnt; uint8_t cnt;}; //--Управляющая структура для работы с hive.blink;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//				КЛАСС RT_HW_BOARDS
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_CONSOLE{
public:					
//----- Параметры -----------------------------------------------------------------------
#pragma pack(push, 4)	//--Установка выравнивания до 4 байт
struct {uint32_t timePeriod; 
        uint32_t ptrBuff;
		uint32_t buff32;
		uint32_t val;
		uint16_t buff16;
		char 	 buffChar;
		uint8_t  buff8; 
		int8_t   index;
	    uint8_t  dir:3, ready:1, head:1, mess:1, resetHead:1, blink:1;
		uint8_t  blinkCnt;  
		} hive; 	   
#pragma pack(pop)		//--Восстановление предыдущей настройки		
//---------------------------------------------------------------------------------------
//		Базовые методы 
//---------------------------------------------------------------------------------------
RT_HW_CONSOLE(){hive.dir=0;};							//--Конструктор
void 	genTick(); 										//--Генератор меток времени(меандр);
bool    eventID  (RT_HW_CONSOLE_EVENT_ID &id, uint8_t ust);	//--Генератор событий - кратность=50мс;
//---------------------------------------------------------------------------------------
//		Утилиты для вывода переменной int32  в различных форматах
//---------------------------------------------------------------------------------------
void  	setVar_ID		(RT_HW_CONSOLE_VAL_ID &iv, char mode, uint8_t form, int32_t val); 	//--Установка параметров выводимой переменной;
void    printVar_ID		(RT_HW_CONSOLE_VAL_ID &iv, uint8_t len, char alig, char suff);		//--Форматированный вывод переменной;
char    getChar_ID		(RT_HW_CONSOLE_VAL_ID &iv, uint8_t n);								//--Получение цифры в символьной форме из переменной;


//---------------------------------------------------------------------------------------
//  	CONSOLE: СЕРВИСНЫЕ ФУНКЦИИ
//--------------------------------------------------------------------------------------- 
char 	boolToChar (bool    val, char sign_0, char sign_1);	//--Конвертор bool в символ;
int32_t floatToInt32 (float val, uint8_t point);			//--Конвертор float в int32_t с умножением на кол-во знаков после точек;
uint8_t getLenBegin(uint8_t len, uint8_t src, char alig);	//--Кол-во заполнителей спереди(alig-> N,L,C,R);   
uint8_t getLenEnd  (uint8_t len, uint8_t src, char alig);	//--Кол-во заполнителей сзади  (alig-> N,L,C,R);    
uint16_t CRC16_String(String &s);							//--Расчет контрольной суммы строковой переменной;
bool    checkChangeInt8x4(uint8_t* ago, uint8_t v1, uint8_t v2, uint8_t v3,uint8_t v4);

//---------------------------------------------------------------------------------------
//  	CONSOLE: ФУНКЦИИ ВЫВОДА НА КОНСОЛЬ
//--------------------------------------------------------------------------------------- 
void 	init(uint32_t speed=115200, bool permit=1, bool headSetMess=1);	//--Настройка порта Serial;
void 	printChar (char val); 											//--Вывод char с контролем готовности консоли;
//---------------------------------------------------------------------------------------
//  	CONSOLE: ФУНКЦИИ ВЫВОДА НА КОНСОЛЬ ЧЕРEЗ ФУНКЦИЮ printChar()
//--------------------------------------------------------------------------------------- 
void 	printEndLine();									//--Выводит окончание строки;
void 	printBlank (uint8_t qnt, char blank=' '); 		//--Выводит символы blank в количестве qnt;
void 	printLine  (uint8_t num=80, char blank='-');	//--Вывод линии;
void	printSuff (char suff);							//--Вывод суффикса: ="Блокирующий суффикс"('~') нет вывода;'Y'-перевод строки; любой другой символ - вывод; 
void 	printTextFromPGM (uint32_t ptr, uint8_t len='80', char alig='C', char blank='s', char suff='~');
void 	printString(String &val, uint8_t len, char alig, char blank, char suff); //--Вывод  String,параметры:val,        len,alig,cr,blank; 
void    printInt8x4(RT_HW_CONSOLE_VAL_ID &iv, uint8_t* ago, uint8_t qnt);

//void    printTestPGM(uint32_t ptr); 
void    PrintInt32	(RT_HW_CONSOLE_EVENT_ID &id, RT_HW_CONSOLE_VAL_ID &iv, uint32_t ptr, int32_t &ago, int32_t val, char mode, uint8_t form,  uint8_t ustPeriod, uint8_t len, char alig, char blank, char suff);
void    PrintFloat  (RT_HW_CONSOLE_EVENT_ID &id, RT_HW_CONSOLE_VAL_ID &iv, uint32_t ptr, float &ago, float val,                uint8_t point, uint8_t ustPeriod, uint8_t len, char alig, char blank, char suff);
void 	PrintString (RT_HW_CONSOLE_EVENT_ID &id, String &s, uint16_t &crc16,                                                                 uint8_t period, uint8_t len, char alig, char blank, char suff);
void    PrintInt8x4 (RT_HW_CONSOLE_EVENT_ID &id, RT_HW_CONSOLE_VAL_ID &iv, uint32_t ptr, uint8_t* ago, uint8_t ustPeriod, uint8_t len, char suff, uint8_t qnt, uint8_t v1, uint8_t v2, uint8_t v3, uint8_t v4);

//void    PrintTextFromPGM(uint32_t ptr, uint8_t len, char alig, char blank, char suff);



//void    PrintString (String &s, uint16_t &crc, uint16_t period, uint8_t len, char alig, char blank, char suff);
//      PrintInt16(RT_HW_CONSOLE_EVENT_ID &id, RT_HW_CONSOLE_VAL_ID &iv, uint32_t ptr, int32_t &ago, int32_t val, char mode, uint8_t form, uint16_t period, uint8_t len, char alig, char blank, char suff)
//=======================================================================================																			 
}; //--конец класса
extern RT_HW_CONSOLE	RT_HW_Console;	//----Глобальная ссылка на созданный объект RT_HW_Console
#endif //#ifndef    RT_HW_CONSOLE_h
//=======================================================================================

