	/*** FLProg ************** Библиотека RT_HW_LCD_HD44780 ver2.1 для Arduino ***************
  Вывод на несколько LCD типа HD44780 переменных String, char, int16_t, float в указанное место указанного дисплея.
  Если задана длина (len) выводимого сообщения, то вывод осуществляется в пределах указанной длины или до конца  строки.
  Если len=0, то вывод осуществляется до конца строк. 
  Сообщение может быть отформатировано(alig) в пределах заданной длины  влево ('L'), по центру ('C'), право ('R').
    При этом к сообщению добавляются пробелы или оно обрезается. Если alig='N' (или другой отличный символ 
	кроме 'L','C','R'), то форматирование не осуществляется. При этом надо иметь ввиду что более короткое сообщение
	не затрет предыдущее более длинное сообщение. Поэтому рекомендуется явно задавать вид форматирования или использовать
    форматирование по умолчанию, которое в библиотеке ='L' (влево).
  Имеется функция вывода мерцающего символа по указанному адресу.	
	
  Библиотеки  разрабатываются с учетом особенностей использования в системе визуального программирования FLProg.
  ecoins@mail.ru 29.09.2019г.
**************************************************************************************************/
//#################################################################################################
#ifndef RT_HW_LCD_HD44780_h
#define RT_HW_LCD_HD44780_h
//-------------------------------------------------------------------------------------------------
#if defined(ARDUINO) && ARDUINO >= 100 
#include <Arduino.h>
#else
#include <WProgram.h>
#endif
#include <stdio.h>  		//-- Библиотека структур C++
//-------------------------------------------------------------------------------------------------
#include <Wire.h>
#include <SPI.h>
//-------------------------------------------------------------------------------------------------
#define RT_HW_LCD_HD44780_VERSION 		23			//--Номер версии библиотоеки
#define RT_HW_LCD_HD44780_MAX_DEVICE  	8			//--Максимальное кол-во устройств
#define RT_HW_LCD_HD44780_MAX_COLS  	20			//--Максимальное кол-во колонок
#define RT_HW_LCD_HD44780_MAX_ROWS  	7			//--Максимальное кол-во строк
#define RT_HW_LCD_HD44780_PERIOD_TICK  	50			//--Период системного времени в ms;
#define RT_HW_LCD_HD44780_MILLIS		millis()	//--Время в миллисекундах;
#define RT_HW_LCD_HD44780_MICROS		micros()	//--Время в микросекундах;
#define RT_HW_LCD_HD44780_DELAY_MCS(x)	delayMicroseconds(x) //--Задержка в микросекундах;
#define RT_HW_LCD_HD44780_BLANK		    ' '			//--Символ заполнения;
#define RT_HW_LCD_HD44780_BREAK_CHAR	'~'			//--Символ "Запрещающего" символа;
#define RT_HW_LCD_HD44780_CODE_BLANK	's'			//--Символ указатель, что заполнитель "пробел";
#define RT_HW_LCD_HD44780_PERIOD_TICK   50			//--Период системного времени в ms;
#define RT_HW_LCD_HD44780_UPPER  		0			//--=0 -нижний регистр, =1 - верхгий регистр;

#define _RT_HW_LCD_HD44780_BYTE_PGM(a)   const uint8_t  a[]  PROGMEM  //--Создание в PROGMEM массива int8_t c именем а произвольной длины 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//         СТРУКТУРЫ
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_LCD_HD44780_FIND_I2C{uint16_t period; uint8_t step; uint8_t cnt;};
struct RT_HW_LCD_HD44780_EVENT	 {uint8_t  blink:1,cnt:7; uint8_t cntBlink; uint32_t timeStart;};
struct RT_HW_LCD_HD44780_REBOOT	 {uint16_t run:1, ago:1, cnt:14;};
struct RT_HW_LCD_HD44780_STEP	 {uint8_t  init; 
								  uint8_t  sendExtra;
								  uint8_t  send;
								  uint8_t  direct;};
struct RT_HW_LCD_HD44780_OK      {uint16_t custom:1, ban:1, find:1, init:1, send:1, sendDevice:1, loadSign:1, cmnd:1, tetrade:1, permit:1, busy:1, link:1;}; 
struct RT_HW_LCD_HD44780_VAL_ID  {uint32_t val; uint16_t len:4, num:4, first:1, mode:2, minus:1, point:2, pnt:1, fresh:1;}; //--Структура для выводимой переменной

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                    СТРУКТУРА ДЛЯ ХРАНЕНИЯ ПАРАМЕТРОВ УСТРОЙСТВА
// mode режим подключении =2 через i2c; =3 через SPI; =4 через пины по 4-х битной шине; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct  RT_HW_LCD_HD44780_DEVICE {  //=34 байт		  
public: 
uint32_t timeDelay;					//-4--Рабочий регистр;
RT_HW_LCD_HD44780_FIND_I2C	find; 	//-4--Регистры для программы поиска устройства на шине i2c;
RT_HW_LCD_HD44780_EVENT		event;	//-6-Регистры для работы с периодическим вызовом;
RT_HW_LCD_HD44780_REBOOT	reboot;	//-2-Регистры для перезапуска lcd;
RT_HW_LCD_HD44780_STEP 	 	step;	//-4--Номера шагов для программ управления дисплеем; 
RT_HW_LCD_HD44780_OK		ok;	  	//-2--Флаги состояний;
uint16_t cols:6,rows:3,num:4,mode:3;//-2--Кол-во колонок и строк дисплея, номер устройства; 											
int8_t	 adr;	 					//-1--i2c:адрес на i2c;     SPI: пин cs;          прямое: пин RS;
uint8_t  work,val,data;				//-3--i2c:счетчик для find; SPI: не используется; прямое: пин E; send: буфер для символа
int8_t   buff[6];					//-6-буфера для регистров при подключении по 4-битной шине;
uint16_t qntReboot;     			//-2--Уставка перезагрузки в тиках=5сек;
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                    СТРУКТУРА ДЛЯ ОБРАБОТКИ ПЕРЕМЕННЫХ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++								 							 
struct RT_HW_LCD_HD44780_VAL    {//uint32_t timePeriod; 
								 uint32_t ptrBuff; uint32_t buff32, val; uint16_t buff16;
								 char 	 buffChar; uint8_t  buff8;  uint8_t  buff8_add; int8_t   index;};			
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//	СТРУКУТУРА УПРАВЛЕНИЯ ДЛЯ БЛОКОВ ВЫВОДА 15 байт
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++    							  		 
struct RT_HW_LCD_HD44780_DIRECT{
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт	
uint8_t  step; 		//--Номер шага в функциях вывода;
uint8_t  dir;		//--Номер состояния для блока FLProg;
uint8_t  num;		//--Номер устройства;
uint8_t  index;		//--Рабочий индекс; 
uint8_t  cnt; 		//--Cчетчик отсчета времени (в тиках);
char	 buff; 		//--Буфер для символа/длины переменной;
uint8_t  col;		//--Номер столбца;
uint8_t  len;		//--Длина выводимого поля; 
uint16_t permit:1, out:1, ago:1, first:1, start:1, period:1, fresh:1, run:1, clean:1, agoEN:1, cntEN:2, agoBool:1; //, EN:1;
uint16_t row:3, head:6,  sz:6, end:1; 			//--Длина заголовка, переменной, суффикса; 
#pragma pack(pop)		//--Восстановление предыдущей настройки
};	  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                    СТРУКТУРА ДЛЯ ХРАНЕНИЯ ПАРАМЕТРОВ ИСПОЛЬЗУЕМЫХ ВСЕМИ УСТРОЙСТВАМИ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
struct RT_HW_LCD_HD44780_device	{uint16_t set;   //-2+1+1+8+4=16
								 uint8_t  qnt:4, i2c_ok:1, spi_ok:1;  //--Кол-во зарегистрированных устройств;
								 uint8_t  buff8; 
								 uint8_t  buffSend[8]; 
								 uint32_t timePeriod;};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                     КЛАСС УПРАВЛЕНИЯ LCD
// device.set  -регистр флагов инициализации устройства. Номер бита соответствует состоянию устроства с этим номером. Могут используются устройства от 1 до 15.
// device.qnt  -кол-во подключенных устройства;
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class RT_HW_LCD_HD44780{
enum {CMND=0,DATA=1};
public:
// ****** ДАННЫЕ *********************************************
#pragma pack(push, 4)			//--Установка выравнивания до 4 байт
RT_HW_LCD_HD44780_DEVICE 		*prt[RT_HW_LCD_HD44780_MAX_DEVICE];	//--Массив указателей на ID устройств
//RT_HW_LCD_HD44780_DEVICE 		nullDevice;
RT_HW_LCD_HD44780_device 		device;								//--Общие регистры для работы с устройствами;
RT_HW_LCD_HD44780_VAL			hive;								//--Общие регистры для преобразования переменных;
#pragma pack(pop)		//--Восстановление предыдущей настройки							
// ****** МЕТОДЫ ********************************************
RT_HW_LCD_HD44780(); //--Конструктор 
//---
bool getBusy(uint8_t num)			 {return prt[num]->ok.busy;};
//------Инициализация параметров;
void	setID_device(RT_HW_LCD_HD44780_DEVICE &z, int8_t mode, int8_t adr=-1,uint8_t cols=16,uint8_t rows=2); //--Установка ID устройства;
bool	setID_direct(RT_HW_LCD_HD44780_DIRECT &id,int8_t  num,uint8_t col,   uint8_t row,    uint8_t len=1);  //--Установка ID блока вывода;
//------Управление выводом устройством
void	direct		(RT_HW_LCD_HD44780_DEVICE &z, uint16_t periodReboot=200);								  //--Управление устройством;
//------Вывод на дисплей---------------------------------------------------------------------------
bool 	printClear	(RT_HW_LCD_HD44780_DIRECT &id);							//--Очистка поля;
bool    printChar   (RT_HW_LCD_HD44780_DIRECT &id, char val);				//--Вывод символа;
bool    printString (RT_HW_LCD_HD44780_DIRECT &id, String  &str, char alig);//--Вывод строковой переменной; 
bool 	printVal	(RT_HW_LCD_HD44780_DIRECT &id, RT_HW_LCD_HD44780_VAL_ID &iv, String &head, char suff, char alig); //--Вывод переменной;
//------Вывод на дисплей череp блоки FLProg---------------------------------------------------------------------------
void 	PrintChar	(RT_HW_LCD_HD44780_DIRECT &id,   uint8_t col, uint8_t row, uint8_t period, uint8_t lcd, bool   EN, bool blink, char val);
void 	PrintBool	(RT_HW_LCD_HD44780_DIRECT &id, 	 uint8_t col, uint8_t row, uint8_t period, uint8_t lcd, bool   EN, bool blink, bool val, char sign0, char sign1);
void 	PrintBlink	(RT_HW_LCD_HD44780_DIRECT &id, 	 uint8_t col, uint8_t row, uint8_t period, uint8_t lcd, bool   EN,                       char sign0, char sign1);
void 	PrintVal	(RT_HW_LCD_HD44780_DIRECT &id,RT_HW_LCD_HD44780_VAL_ID &iv,int32_t &ago,  String &head, char alig, uint8_t len,
													 uint8_t col, uint8_t row, uint8_t period, uint8_t lcd, bool   EN, bool blink, int32_t val,char mode, uint8_t form, char suff);
void 	PrintFloat	(RT_HW_LCD_HD44780_DIRECT &id, RT_HW_LCD_HD44780_VAL_ID &iv, float &ago,  String &head, char alig, uint8_t len, 
													 uint8_t col, uint8_t row, uint8_t period, uint8_t lcd, bool   EN, bool blink, float val, uint8_t point, char suff);	
void 	PrintString	(RT_HW_LCD_HD44780_DIRECT &id,   uint16_t &crc16,                                       char alig, uint8_t len,     
                                                     uint8_t col, uint8_t row, uint8_t period, uint8_t lcd, bool   EN, bool blink, String &str);														 
void 	PrintTxt	(RT_HW_LCD_HD44780_DIRECT &id,                                                          char alig, uint8_t len,                                                    
													 uint8_t col, uint8_t row, uint8_t period, uint8_t lcd, bool   EN, bool blink, String &str);														 
//------Вспомогательные функции---------------------------------------------------------------------------------------
bool    printBlank	(RT_HW_LCD_HD44780_DIRECT &id); 						//--Вывод пробелов в кол-ве установленных в is.index;
void    printBegin	(RT_HW_LCD_HD44780_DIRECT &id);							//--Функция начала вывода;
void    printEnd	(RT_HW_LCD_HD44780_DIRECT &id);							//--Функция завершения вывода;
void	setVar_ID	(RT_HW_LCD_HD44780_VAL_ID &iv, char mode, uint8_t form, int32_t val); //--Обработка численной переменной;
char	getChar_ID	(RT_HW_LCD_HD44780_VAL_ID &iv, uint8_t n);				//--Получение символа из числа;
uint8_t getLenString(RT_HW_LCD_HD44780_DIRECT &id, String &str);			//--Получение длины строковой переменной;
uint16_t CRC16_String(String &s);											//--Расчет контрольной суммы строковой переменной;
char    boolToChar 	(bool val, char sign_0, char sign_1);					//--Конвертор переменной bool в char; 	
int32_t floatToInt32(float val, uint8_t point);								//--Конвертор float в int32_t с умножением на кол-во знаков после точек;
bool 	checkDirect (RT_HW_LCD_HD44780_DIRECT &id); 						//--Контроль готовности устройства;
//------Управление устройством---------------------------------------------------------------------
void    init     	(RT_HW_LCD_HD44780_DEVICE &z);							//--Инициализация LCD;
bool	send  		(RT_HW_LCD_HD44780_DEVICE &z, uint8_t val, uint8_t cmnd=DATA, uint16_t time=0, uint8_t tetrade=0);	 //--Отправка в LCD данных;
bool    send_i2c    (RT_HW_LCD_HD44780_DEVICE &z);
bool    send_bus4   (RT_HW_LCD_HD44780_DEVICE &z);
bool    send_spi    (RT_HW_LCD_HD44780_DEVICE &z);
bool    send_595	(RT_HW_LCD_HD44780_DEVICE &z);
void    spi_595     (RT_HW_LCD_HD44780_DEVICE &z, uint8_t val, bool En);
void    pin_595     (RT_HW_LCD_HD44780_DEVICE &z, uint8_t val, bool En);
//--Служебные функции------------------------------------------------------------------------------
bool    i2c_find(uint8_t &step, uint8_t &cnt, uint16_t &period, uint32_t &timeDelay, int8_t adr, uint8_t cntMax); //--Поиск устройства на шине i2c;	  
uint8_t	getFistBitUint16(uint16_t &reg);									//--Возвращает первый не занятый бит из регистра uint16_t; 
void 	genDeviceTick	(RT_HW_LCD_HD44780_DEVICE &z);						//--Генератор вывода меток времени (меандры) устройства;
bool	genDeviceReboot	(RT_HW_LCD_HD44780_DEVICE &z, uint16_t period);		//--Генератор перезагрузки устройства (от меток времени устройства);
void	genEventDir		(RT_HW_LCD_HD44780_DIRECT &id, uint8_t period, bool EN=1, bool blink=0);
uint8_t pointCursor  (uint8_t col, uint8_t row);							//--Вычисление позиции курсора в памяти Lcd	
//-------------------------------------------------------------------------------------------------
uint8_t getLenBegin(uint8_t len,uint8_t src,char alig);						//--Возвращает кол-во символов, которые надо добавить спереди;    
uint8_t getLenEnd  (uint8_t len,uint8_t src,char alig);						//--Возвращает кол-во символов, которые надо добавить сзади;
//bool 	createChar   (uint8_t location, uint8_t charmap[], uint8_t lcd=0);	//--Загрузка до 8 символов в CGRAM  ?? Зарезервировано для расширения
//bool    loadChar     (uint8_t char_num, uint8_t *rows,     uint8_t lcd=0);	//  ??Зарезервировано для расширения 
};//==============  Конец класса ==================================================================
extern RT_HW_LCD_HD44780 RT_HW_Lcd_hd44780; //--Создание объекта LCD типа HD44780
#endif
//=================================================================================================