#include "RT_HW_MCP23017.h"
//-------------------------------------------------------------------------------------------------
//          Параметры - адреса для IOCON.BANK=0
//-------------------------------------------------------------------------------------------------
#define MCP_BASE	0x20	//начало адресного пространства MCP23017 (на шине i2c)
#define MCP_IODIRA	0x00	//регистр направления пинов A: =0x00(default) - все на выход, 0xff - все на вход
#define MCP_IODIRB	0x01	//регистр направления пинов B: =0x00(default) - все на выход, 0xff - все на вход
#define MCP_IPOLA	0x02	//регистр инверсии    пинов A: =0x00(default) - все прямые,   0xff - все инверсные
#define MCP_IPOLB	0x03	//регистр инверсии    выводов регистра B: =0x00(default) - все прямые,   0xff - все инверсные
#define MCP_GPPUA	0x0C	//регистр подтяжки входов регистра A резисторами 100кОм: =0x00(default) - не подтянуты,   0xff - все подтянуты
#define MCP_GPPUB	0x0D	//регистр подтяжки входов регистра A резисторами 100кОм: =0x00(default) - не подтянуты,   0xff - все подтянуты
#define MCP_GPIOA	0x12	//регистр выводов  на пины A
#define MCP_GPIOB	0x13	//регистр выводов  на пины B
#define MCP_IOCON   0x0A	//регистр конфигурации  
//-------------------------------------------------------------------------------------------------
#define DEVICE_MAX_TIME_READ    2		//--Максимальное время на считывание регистров
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//		КОНСТРУКТОР
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
RT_HW_MCP23017:: RT_HW_MCP23017(){
  hive.permit_bus=hive.i2c_ok=hive.spi_ok=false; 	//--Очистка флага инициализации используемых интерфейсов (i2c,SPI);
  hive.busyTick=0;	//--Очистка флага  захвата генератора тиков;
  hive.timePeriod=RT_HW_MCP23017_MILLIS;}; 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		ИНИЦИАЛИЗАЦИЯ И УПРАВЛЕНИЕ УСТРОЙСТВОМ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MCP23017:: direct(RT_HW_MCP23017_DEVICE &z,  uint8_t vA, uint8_t vB){
 if(z.ok.permitTick) {genTick();}			//--Вызов "Генератора", если устройство захватило его; 
 // genTick();
switch(z.dir){default: z.dir=0;
  //------Настройка параметров устройства---------------------------------------------
  case 0: if(!z.ok.custom) break;			//--Ожидание настройки параметров устройства;
		  if(z.ok.ban)     break;   		//--Если устройство не может быть использовано;
		  if(!hive.busyTick) {hive.busyTick=true; z.ok.permitTick=true;} //--Захват и инициализация "Генератора временных событий занят";
		  z.dir++;				
  //------Поиск устройства на шине i2c ------------------------------------------------
  case 1: if(!bitRead(z.tuning,6)) {z.ok.find=i2c_find(z.workFind, z.cnt, z.periodFind, z.timeFind, z.adr, 20); if(!z.ok.find) break;} 
	      z.ok.link=true; z.cnt=0; z.dir++; 
  //------Инициализация устройства-----------------------------------------------------
  case 2: init(z); 
		  if(!z.ok.link){z.dir=1; break;}	//--Если потеряна связь, переход повторный поисл;
		  z.ok.permit=true;					//--Установка флага разрешение работы устройства; 
		  z.dir++; 
  //------Рабочий режим---------------------------------------------------------------		
  case 3: if(!z.ok.link)	{z.dir=1; z.ok.permit=0;  break;}		 //Переход, если взведен флаг "Потеря связи";
		  if(z.ok.agoBlink!=hive.blink){z.ok.agoBlink=hive.blink; if(++z.cnt>z.period) {z.cnt=1; z.ok.runDevice=1;}}
		  if(z.ok.runDevice) {z.ok.runDevice=false;					//--Обмен с устройством по флагу z.ok.runDevice;
			if(z.outA!=vA) {z.outA=vA; z.ok.runOutA=true;}			
			if(z.outB!=vB) {z.outB=vB; z.ok.runOutB=true;}	
		    if(!bitRead(z.tuning,7))  {z.ok.runOutA= z.ok.runOutB=true;}
			//z.ok.runOutA= z.ok.runOutB=true;
			device(z); }			//--Обмен с данными с устройством;
		  //---
		  break;
}
}; // END direct(RT_HW_LCD_HD44780_DEVICE &z)		

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
// 			        УСТАНОВКА ID УСТРОЙСТВА 
//adr 		интерфейс i2c -адрес устройства на шине;  интерфейс SPI -адрес пина cs; 
//tuning:	bit0: режим регистра А      0/1 ->ввод/вывод;
//			bit1: режим регистра В      0/1 ->ввод/вывод;		
//			bit2: инверсия региc.А		0/1 ->без инверсии/ с инверсией;
//			bit3: инверсия регис.В 		0/1 ->без инверсии/ с инверсией;	
//			bit4: входа регистра А		0/1 ->без подтяжки/ с подтяжкой к Vcc;	
//			bit5: входа регистра В		0/1 ->без подтяжки/ с подтяжкой к Vcc;
//			bit6: интерфейс подкл.		0/1 ->i2c/SPI;	
//			bit7: режим записи 			0/1 ->в каждом цикле/по изменениям;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void RT_HW_MCP23017:: setID_device(RT_HW_MCP23017_DEVICE &z, int8_t adr, char modeBus, char modeA, char modeB, uint8_t period){
  if(z.ok.custom)  return; 	z.ok.custom=1;	//--Выход, если устройство инициализировано или установка флага "Параметры устройства установлены";
 z.tuning=0;
  switch(modeA){default:
	  case 'R': bitSet(z.tuning,2); bitSet(z.tuning,4); break;
	  case 'r': break;
	  case 'w': bitSet(z.tuning,0); break;
	  case 'W': bitSet(z.tuning,0); bitSet(z.tuning,7); break;	 	  
  }
   switch(modeB){default:
	  case 'R': bitSet(z.tuning,3); bitSet(z.tuning,5); break;
	  case 'r': break;
	  case 'w': bitSet(z.tuning,1); break;
	  case 'W': bitSet(z.tuning,1); bitSet(z.tuning,7); break;	 	  
  } 
  if(modeBus=='s') bitSet(z.tuning,6); 
  z.period=period;
  z.step.mode=z.tuning & 0x03;
  //--Обработка адреса
  if((adr<0) || (adr>127)) {z.ok.ban=1; return;}
  if((adr>=0) && (adr<=7)) {z.adr=MCP_BASE+adr;} else {z.adr=adr;}	//--Установка адреса: если адрес от 0-7, то это адрес в адресном пространстве MCP_BASE 
  //---Настройка интерфейса 
  if(hive.permit_bus) { 		//--проверка разрешения инициализации интерфейсов (для поддержания внешней инициализации)
  if(bitRead(z.tuning,6)) {pinMode(z.adr,OUTPUT); digitalWrite(z.adr,true); if(!hive.spi_ok){hive.spi_ok=true; SPI.begin(); }}	//--Настройка интерфейса SPI; 		
					 else {                                                 if(!hive.i2c_ok){hive.i2c_ok=true; Wire.begin();}}; //--Настройка интерфейса i2c;
  }
};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//		ИНИЦИАЛИЗАЦИЯ УСТРОЙСТВА
//adr   i2c:адрес устройства на шине;  SPI:адрес пина CS; 
//mode:	bit0: режим регистра А      0/1 ->ввод/вывод;
//		bit1: режим регистра В      0/1 ->ввод/вывод;		
//		bit2: инверсия региc.А		0/1 ->без инверсии/ с инверсией;
//		bit3: инверсия регис.В 		0/1 ->без инверсии/ с инверсией;	
//		bit4: входа регистра А		0/1 ->без подтяжки/ с подтяжкой к Vcc;	
//		bit5: входа регистра В		0/1 ->без подтяжки/ с подтяжкой к Vcc;
//		bit6: интерфейс подкл.		0/1 ->i2c/SPI;	
//		bit7: режим записи 			0/1 ->в каждом цикле/по изменениям;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MCP23017:: init(RT_HW_MCP23017_DEVICE &z) {
                          hive.buff[0]=MCP_IODIRA; 						//--Адрес регистра, начиная с которого будут записаны четыре параметра
  if(bitRead(z.tuning,0)) hive.buff[1]=0x00; else hive.buff[1]=0xff;	//--Установка направление регистра A (ввод=0xff/вывод=0x00);
  if(bitRead(z.tuning,1)) hive.buff[2]=0x00; else hive.buff[2]=0xff;	//--Установка направление регистра В (ввод=0xff/вывод=0x00);
  if(bitRead(z.tuning,2)) hive.buff[3]=0xff; else hive.buff[3]=0x00;	//--Установка инверсии    регистра A (без инверсии=0x00/с инверсией=0xff);
  if(bitRead(z.tuning,3)) hive.buff[4]=0xff; else hive.buff[4]=0x00;	//--Установка направление регистра В (без инверсии=0x00/с инверсией=0xff);  
						  hive.qnt=5;		 if(!write(z)) return;    	//--Вывод и выход по ошибке;
						  hive.buff[0]=MCP_GPPUA; 						//--Адрес регистра, начиная с которого будут записаны два параметра.
  if(bitRead(z.tuning,4)) hive.buff[1]=0xff; else hive.buff[1]=0x00;	//--Установка подтяжжки входов регистра A (без подтяжки =0x00/с подтяжкой=0xff);
  if(bitRead(z.tuning,5)) hive.buff[2]=0xff; else hive.buff[2]=0x00;	//--Установка подтяжжки входов регистра B (без подтяжки =0x00/с подтяжкой=0xff);
  						  hive.qnt=3;		 if(!write(z)) return;		//--Вывод и выход по ошибке;
};					
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//            			 ОБМЕН С УСТРОЙСТВОМ
//mode:	bit0: режим регистра А      0/1 ->ввод/вывод;
//		bit1: режим регистра В      0/1 ->ввод/вывод;	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void RT_HW_MCP23017:: device(RT_HW_MCP23017_DEVICE &z){
if(!z.ok.permit) return;	
	switch(z.step.mode){
		//----Регистр А - ввод, регистр В - ввод---------------------------------------------------
		case 0:	hive.buff[0]=MCP_GPIOA;	hive.qnt=2;				//--Транзакция для чтения регистров A и B	
				if(!read(z)) 							break;	//--Чтение и выход если ошибка обмена;
				z.inA=hive.buff[0]; z.inB=hive.buff[1];	break;	//--Сохранение входов регистров A и B и выход;
		//----Регистр A- вывод, регистр B - ввод---------------------------------------------------
		case 1: if(z.ok.runOutA){z.ok.runOutA=false;			//--Обработка флага запуска вывода в регистр A;
				hive.buff[0]=MCP_GPIOA;	hive.buff[1]=z.outA;  hive.qnt=2;	//--Транзакция для вывода регистра;		
				if(!write(z)) {return;};}						//--Вывод и выход если ошибка обмена;						 
				hive.buff[0]=MCP_GPIOB; hive.qnt=1;				//--Транзакция для чтения регистра B		
				if(!read(z)) 							break;	//--Выход по ошибке обмена;
				z.inB=hive.buff[0];						break;	//--Сохранение входов регистра B и выход;
		//----Регистр А- ввод, регистр B - вывод---------------------------------------------------
		case 2: if(z.ok.runOutB){z.ok.runOutB=false;			//--Обработка флага запуска вывода в регистр A;
				hive.buff[0]=MCP_GPIOB;	hive.buff[1]=z.outB;  hive.qnt=2;	//--Транзакция для вывода регистра;		
				if(!write(z)) {return;};}						//--Вывод и выход если ошибка обмена;					 
				hive.buff[0]=MCP_GPIOA; hive.qnt=1;				//--Транзакция для чтения регистра B		
				if(!read(z))							break;	//--Выход по ошибке обмена;
				z.inA=hive.buff[0];						break;	//--Сохранение входов регистра B и выход;				
		//--Регистр А - вывод; Регистр В - вывод----------------------------------------------------
		case 3:	if((!z.ok.runOutA) && (!z.ok.runOutB))	break;  z.ok.runOutA=z.ok.runOutB=false; //--Обработка флагов вывода в регистры;				
				hive.buff[0]=MCP_GPIOA;  hive.buff[1]=z.outA;  hive.buff[2]=z.outB; hive.qnt=3; //--Транзакция для вывода регистров А и В;
				write(z);    							break;	//--Вывод и выход.
}};	
//-------------------------------------------------------------------------------------------------
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
// 		Запись регистров в устройство
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
bool RT_HW_MCP23017:: write(RT_HW_MCP23017_DEVICE &z){
  if(hive.qnt>RT_HW_MCP23017_SIZE_BUFF ) hive.qnt=RT_HW_MCP23017_SIZE_BUFF ; //--Защита от доступа за пределом буфера;
  if(bitRead(z.tuning,6)){//---Запись в SPI регистров из буфера hive.buff
						  digitalWrite(z.adr,0); 
						 // SPI.transfer(hive.buff,hive.qnt);  
						  digitalWrite(z.adr,0); return 1;
						  }	//SPI
					else {//---Запись в i2c регистров из буфера hive.buff 
						  Wire.beginTransmission(z.adr); Wire.write(hive.buff,hive.qnt); 										//-i2c
						  if(Wire.endTransmission()!=0) {z.ok.link=false; return 0;} return 1;}};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
// 		Чтение региcтров из устройства
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
bool RT_HW_MCP23017:: read(RT_HW_MCP23017_DEVICE &z){ 
 if(hive.qnt>RT_HW_MCP23017_SIZE_BUFF ) hive.qnt=RT_HW_MCP23017_SIZE_BUFF ; //--Защита от доступа за пределом буфера;
 if(bitRead(z.tuning,6)) {//--Чтение регистров из SPI - результат в буфере hive.buff
						  digitalWrite(z.adr,0); 
						  //SPI.transfer(hive.buff,hive.qnt); 
						  SPI.transfer(hive.buff[0]); 
						  digitalWrite(z.adr,0); 
						  return 1;}//SPI
					else {//--Чтение регистров из i2c - результат в буфере hive.buff
						  Wire.beginTransmission(z.adr); 	//--Установка связи с устройством;
						  Wire.write(hive.buff[0]); 		//--Отправка адреса первого читаемого регистра;
					      hive.buff8=Wire.endTransmission();
						  if(Wire.endTransmission()!=0) {z.ok.link=false; return 0;}; //--Выход по ошибке обмена;
						  Wire.requestFrom((uint8_t)z.adr,(uint8_t)hive.qnt); //--Запрос на чтение регистров; 
					      hive.buff8=0; 
						  hive.timeStart=RT_HW_MCP23017_MILLIS+DEVICE_MAX_TIME_READ;  //--Установка таймера тайм-аута с защитой от переполнения;
						  if(RT_HW_MCP23017_MILLIS>hive.timeStart) hive.timeStart=RT_HW_MCP23017_MILLIS+DEVICE_MAX_TIME_READ;	
						     if(RT_HW_MCP23017_MILLIS>hive.timeStart) hive.timeStart=RT_HW_MCP23017_MILLIS+DEVICE_MAX_TIME_READ;							  
						while(Wire.available())    //--едомое устройство может послать меньше, чем запрошено
							{ hive.buff[hive.buff8]=Wire.read();
							  hive.buff8++;
						    }
						    return z.ok.link;
						  }
};	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                  ПОИСК УСТРОЙСТВА НА ШИНЕ i2c 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
bool RT_HW_MCP23017:: i2c_find(uint8_t &step, uint8_t &cnt, uint16_t &period, uint32_t &timeDelay,  int8_t adr, uint8_t cntMax){
switch(step){default: step=0;   
  case 0: if(adr<0)	return 0;; 										//--Выход, если не допустимый адрес;
		  cnt=0; period=0; 	timeDelay=RT_HW_MCP23017_MILLIS; step++; 	//--Сброс параметров, запоминаем время, переход;
  case 1: if(!((RT_HW_MCP23017_MILLIS-timeDelay)>period)) return 0; timeDelay=RT_HW_MCP23017_MILLIS; //--Ожидание задержки по времени;
		  cnt++; if(cnt>=cntMax) cnt=cntMax; 						//--Ограничение счетчика сверху;
		  Wire.beginTransmission(adr);								//--Запрос на шину i2c
			if(Wire.endTransmission()==0){ 							//		 и проверка кода ответа (=0 - устройство ответило);
			if(cnt <2) {period=50;			return 0;}				//--Выход для повторного поиска (для надежности) или успешное завершение;
			if(cnt==2) {step=0; 			return 1;}				//--
			if(cnt >2) {period=50;   cnt=1; return 0;}}				//--Успешное завершение;
		  period =100*cnt;					return 0;}};			//--Выход для повторного поиска
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//	 ГЕНЕРАТОР ВРЕМЕННЫХ СОБЫТИЙ И МЕАНДРА ДЛЯ УПРАВЛЕНИЯ УСТРОЙСТВОМ 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
void RT_HW_MCP23017:: genTick(){hive.tick=hive.tick500=false;	
  if(RT_HW_MCP23017_MILLIS>=hive.timePeriod) {hive.timePeriod=RT_HW_MCP23017_MILLIS+RT_HW_MCP23017_PERIOD_TICK; 
    hive.tick  =true; hive.blink  =!hive.blink; if(++hive.cntTick> 50){hive.cntTick=1; hive.tick500  =true; hive.blink500  =!hive.blink500;}}
	};
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
RT_HW_MCP23017 RT_HW_mcp23017;//--Создание внешнего объекта	
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


/*
switch(z.stepDevice){default: z.stepDevice=0;						
	//----ПРОВЕРКА ВНЕШНЕГО ФЛАГА ЗАПУСКА РАБОТЫ С УСТРОЙСТВОМ ДЛЯ НАЧАЛА ОБМЕНА С УСТРОЙСТВОМ 
	case 0:	z.rdA=z.dirA; z.rdB=z.dirB; z.wrA=!z.dirA; z.wrB=!z.dirB;
			if((z.run==1) && (z.agoA==outA)) z.wrA=0;  
			if((z.run==1) && (z.agoB==outB)) z.wrB=0;  	
			z.stepWork=0; z.stepDevice++;
	case 1: if(!write(z, outA, outB)) return 0; else {z.stepWork=0;  z.stepDevice++;}	//--Запись
	case 2: if(!read (z))             return 0; else {z.stepWork=0;  z.stepDevice++;}	//--Чтение	
	case 3: z.run=0; z.stepDevice=0;  return 1;} //--Завершение обмена
return 0;
*/






//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//			УСТАНОВКА ПАРАМЕТРОВ C УЧЕТОМ СПОСОБА ПОДКЛЮЧЕНИЯ
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
/*
void RT_HW_MCP23017:: setID_device(RT_HW_MCP23017_DEVICE &z, char mode, int8_t adr, char modeA, char modeB, int16_t period){int8_t x;
  if(z.ok.custom)  return; 	z.ok.custom=1;	//--Выход, если устройство инициализировано или установка флага "Параметры устройства установлены";
  z.adr=adr; 								//--i2c: адрес устройства на шине; SPI: адрес пина CS; bus4: номер пина RS LCD; 
  z.mode=mode;
  switch(z.mode){default:    z.ok.ban=1; break;
	  case 'I': if(z.adr<0) {z.ok.ban=1; break;} if(!hive.i2c_ok) {hive.i2c_ok=true; Wire.begin();} break; //----Настройка интерфейса i2c; 		
	  case 'S': if(z.adr<0) {z.ok.ban=1; break;}  pinMode(z.adr,OUTPUT); digitalWrite(z.adr,true); if(!hive.spi_ok) {hive.spi_ok=true; SPI.begin(); 
			  digitalWrite(z.adr,true);} break;}	//----Настройка интерфейса SPI; 				
  z.ok.find=z.ok.init=z.ok.link=z.ok.permit=z.ok.permitTick=false; //--Сброс флагов;
  z.dir=0;
};
*/


//-------------------------------------------------------------------------------------------------
// 			        УСТАНОВКА ID УСТРОЙСТВА 
//adr   i2c: адрес устройства на шине; SPI: адрес пина CS; bus4: номер пина RS LCD;  
//mode:	bit0: интерфейс подключения 0/1 ->i2c/SPI;
//		bit1: режим регистра А      0/1 ->ввод/вывод;
//		bit2: режим регистра В      0/1 ->ввод/вывод;		
//		bit3: инверсия регистра А	0/1 ->без инверсии/ с инверсией;
//		bit4: инверсия регистра В 	0/1 ->без инверсии/ с инверсией;	
//		bit5: входа регистра А		0/1 ->без подтяжки/ с подтяжкой к Vcc;	
//		bit6: входа регистра В		0/1 ->без подтяжки/ с подтяжкой к Vcc;	
//		bit7: режим записи 			0/1 ->в каждом цикле/по изменениям;	
//-------------------------------------------------------------------------------------------------
/*
void RT_HW_MCP23017:: setID_device(RT_HW_MCP23017_DEVICE &z, int8_t adr, int16_t period){
  if(z.ok.custom)  return; 	z.ok.custom=1;	//--Выход, если устройство инициализировано или установка флага "Параметры устройства установлены";
  //--Обработка адреса
  if((adr<0) || (adr>127)) {z.ok.ban=1; return;}
  if((adr>=0) && (adr<=7)) {z.adr=MCP_BASE+adr;} else {z.adr=adr;}	//--Установка адреса: если адрес от 0-7, то это адрес в адресном пространстве MCP_BASE 
  //---Настройка интерфейса 
  if(!hive.permit) {hive.permit=1; 	//--проверка наличия инициализации устройства
  if(bitRead(z.mode,0)) {                                                 if(!hive.i2c_ok) {hive.i2c_ok=true; Wire.begin();}}	//--Настройка интерфейса i2c; 		
				   else {pinMode(z.adr,OUTPUT); digitalWrite(z.adr,true); if(!hive.spi_ok) {hive.spi_ok=true; SPI.begin() ;}}}; //--Настройка интерфейса SPI
};
*/
  
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// 			   ОТПРАВКА КОМАНД И ДАННЫХ В УСТРОЙСТВО
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++//bool RT_HW_MCP23017:: write (RT_HW_MCP23017_DEVICE &z){
//if(bitRead(z.mode,0)){} //--SPI
//			  else  { Wire.beginTransmission(z.adr); 
//			          Wire.write(z.buff,z.qnt);			//--Отправка транзакции в устройство
//			  if(Wire.endTransmission()) {z.ok.link=0; return 1;}}; //--i2c
//};



//-------------------------------------------------------------------------------------------------
//            	ИНИЦИАЛИЗАЦИЯ И КОНТРОЛЬ УСТРОЙСТВА
//-------------------------------------------------------------------------------------------------


/*
bool RT_HW_MCP23017:: begin(RT_HW_MCP23017_DEVICE &z){
switch(z.stepBegin){default: z.stepBegin=0;						
  //----ОЖИДАНИЕ УСТАНОВКИ ПАРАМЕТРОВ УСТРОЙСТВА (выполняется функцией setID_device() )О
  case 0: switch(z.dir){default: z.stepBegin=0; 
			case 0: return 0; //--выход, если параметры устройства z не инициализировались (используется функция setID_device() ) 
			case 1: return 1; //--выход, если после инициализация устройство z (функцией setID_device()), устройство признано не пригодным к использованию.
							  //       		Для данного устройства, это когда адрес меньше 0.                          
			case 2: 
				#ifndef RT_HW_MAIN_h
				Wire.begin();								//--Инициализация шины i2c с параметрами по умолчанию, если не используется библиотека RT_HW_MAIN.h		
				#else
				if(!RT_HW_Wire.init) 						//--Инициализация шины i2c с параметрами по умолчанию, если используется библиотека RT_HW_MAIN.h,	
					{RT_HW_Wire.init=true; Wire.begin();}	//    																но шина i2c не инициализировалась.
				#endif
			z.stepBegin++; break;}
  case 1: if(z.device==3) {if(event_mcs(z.timeStartEvent,DEVICE_BEGIN_PERIOD_EVENT)) {z.event=1;} else {z.event=0;} return 1;} //--Генераторо событий
          z.event=0; z.stepFind=0; z.stepBegin++;											//--Переход после обнаружения устройства 			
  //----ПОИСК УСТРОЙСТВА НА ШИНЕ i2c 
  case 2: find(z.stepFind, z.addr, z.device, z.count, z.timeStart); if(z.device!=2) break;	//--Поиск устройства
		  z.timeStart=millis(); z.stepBegin++; break;										//--Переход после обнаружения устройства
  //----ИНИЦИАЛИЗАЦИЯ НАПРАВЛЕНИЯ И ПОЛЯРНОСТИ(для ввода) РЕГИСТРОВ 
  case 3: if(!event_ms(z.timeStart,DEVICE_I2C_DELAY_RESTART_SHORT)) break;
		  z.data[0x00]=MCP_IODIRA; 							//--Адрес регистра, с которого будут записаны четыре параметра
		  if(z.dirA)	z.data[1]=0xff;	else z.data[1]=0;	//--Установка направление регистра A (ввод/вывод)
		  if(z.dirB) 	z.data[2]=0xff;	else z.data[2]=0;	//--Установка направление регистра В (ввод/вывод)
		  if(z.ipolA) 	z.data[3]=0xff;	else z.data[3]=0;	//--Установка полярности регистров A (при работк на ввод)  
		  if(z.ipolB) 	z.data[4]=0xff;	else z.data[4]=0;	//--Установка полярности регистров A (при работк на ввод)  	
		  Wire.beginTransmission(z.addr); Wire.write(z.data,5);	//--Передача в i2c-slave 5 байт.
		  if(Wire.endTransmission()) {z.device=z.stepBegin=0; return 1;}  z.stepBegin++; //--Выход, если ошибка обмена, или переход на следующий этап
  //----ИНИЦИАЛИЗАЦИЯ ПОДТЯЖКИ РЕГИСТРОВ ВНУТРЕННИМИ РЕЗИСТОРАМИ 100к(для ввода)
  case 4: z.data[0x00]=MCP_GPPUA; 							//--Адрес регистра, с которого будут записаны четыре параметра.
		  if(z.gppuA) 	z.data[1]=0xff;	else z.data[1]=0;	//--Установка направление регистра A (ввод/вывод). 
		  if(z.gppuB) 	z.data[2]=0xff;	else z.data[2]=0;	//--Установка направление регистра В (ввод/вывод).
		  Wire.beginTransmission(z.addr); Wire.write(z.data,3);	//--Передача в i2c-slave 3 байта.
		  if(Wire.endTransmission()) z.device=0; else z.device=3; z.stepBegin=0; return 1;//--Выход с ошибкой или нормальный и завершение.
} return 0;
}; //--End	
*/
//-------------------------------------------------------------------------------------------------
//            			 ОБМЕН С УСТРОЙСТВОМ
//uint8_t  dirA:1, dirB:1, ipolA:1,ipolB:1, gppuA:1, gppuB:1;    
//-------------------------------------------------------------------------------------------------
/*
bool RT_HW_MCP23017:: device1(RT_HW_MCP23017_DEVICE &z, uint8_t &outA, uint8_t &outB){
if(z.device!=3) {z.run=3; z.stepDevice=0;	return 1;}	if (!z.run) return 1;	//--Проверка готовности дисплея(device=3) и команды вывода(run>0).
//-------------------------------------------------------------------------------------------------
switch(z.stepDevice){default: z.stepDevice=0;						
	//----ПРОВЕРКА ВНЕШНЕГО ФЛАГА ЗАПУСКА РАБОТЫ С УСТРОЙСТВОМ ДЛЯ НАЧАЛА ОБМЕНА С УСТРОЙСТВОМ 
	case 0:	z.rdA=z.dirA; z.rdB=z.dirB; z.wrA=!z.dirA; z.wrB=!z.dirB;
			if((z.run==1) && (z.agoA==outA)) z.wrA=0;  
			if((z.run==1) && (z.agoB==outB)) z.wrB=0;  	
			z.stepWork=0; z.stepDevice++;
	case 1: if(!write(z, outA, outB)) return 0; else {z.stepWork=0;  z.stepDevice++;}	//--Запись
	case 2: if(!read (z))             return 0; else {z.stepWork=0;  z.stepDevice++;}	//--Чтение	
	case 3: z.run=0; z.stepDevice=0;  return 1;} //--Завершение обмена
return 0;};
*/

//-------------------------------------------------------------------------------------------------
//                           ЗАПИСЬ В РЕГИСТРЫ A,B (одновременный вызов с read() не допускается
//-------------------------------------------------------------------------------------------------
/*
bool RT_HW_MCP23017:: write(RT_HW_MCP23017_DEVICE &z, uint8_t &outA, uint8_t &outB){	
	//--НАСТРОЙКА ТРАНЗАКЦИИ (первый адрес записываемого регистра, значение регистров(одно или два), кол-во записываемых байт) 
	if( z.wrA &&  z.wrB) {z.data[0]=MCP_GPIOA; if(z.ipolA) {z.data[1]=0xFF-outA;} else {z.data[1]=outA;} 
											   if(z.ipolB) {z.data[2]=0xFF-outB;} else {z.data[2]=outB;} z.qntData=2;} //--Транзакция для записи регистров A и B
	if( z.wrA && !z.wrB) {z.data[0]=MCP_GPIOA; if(z.ipolA) {z.data[1]=0xFF-outA;} else {z.data[1]=outA;} z.qntData=1;}	//--Транзакция для записи в регистр A
	if(!z.wrA &&  z.wrB) {z.data[0]=MCP_GPIOB; if(z.ipolB) {z.data[1]=0xFF-outB;} else {z.data[1]=outB;} z.qntData=1;}	//--Транзакция для записи в регистр B
	if(!z.wrA && !z.wrB) {return 1;}  											//--Выход, если ни один из флагов записи не взведен.
	Wire.beginTransmission(z.addr); Wire.write(z.data,(z.qntData+1));			//--Отправка транзакции в устройство
	if(Wire.endTransmission()) {z.device=0; return 1;}
	z.agoA=outA; z.agoB=outB;
	return 1;}						//--Завершение записи. При ошибке 0->z.device		
*/	
//-------------------------------------------------------------------------------------------------
//                           ЧТЕНИЕ В РЕГИСТРЫ A,B
//-------------------------------------------------------------------------------------------------
/*
bool RT_HW_MCP23017:: read(RT_HW_MCP23017_DEVICE &z){
	//--НАСТРОЙКА ТРАНЗАКЦИИ (первый адрес записываемого регистра) 
	if( z.rdA &&  z.rdB) {z.data[0]=MCP_GPIOA; z.qntData=2;}  	//--Транзакция для чтения регистров A и B							
	if( z.rdA && !z.rdB) {z.data[0]=MCP_GPIOA; z.qntData=1;}	//--Транзакция для вывода регистра A
	if(!z.rdA &&  z.rdB) {z.data[0]=MCP_GPIOB; z.qntData=1;}	//--Транзакция для вывода регистра B
	if(!z.rdA && !z.rdB) {return 1;}							//--Выход, если ни один из флагов ввода не взведен.		
	//----ЗАПИСЬ АДРЕСА ЧТЕНИЯ ДАННЫХ
	Wire.beginTransmission(z.addr); Wire.write(z.data[0]); //--Запись адреса в устройство
	if(Wire.endTransmission())	{z.device=0; z.stepWork=0; return 1;}	//--Завершение отправки адреса. При ошибке выход и 0->z.device	
	//----ОТПРАВКА ЗАПРОСА НА ЧТЕНИЕ
	Wire.requestFrom(z.addr,z.qntData); 	//--Запрос на чтение регистров
	z.timeStart=millis(); z.stepWork++;	//--Установка таймера и переход на следующий этап
	//-----ОЖИДАНИЕ ПРИХОДА ДВУХ БАЙТОВ С ТАЙМ_АУТОМ
	do{if(getPastTime(z.timeStart,millis()) > DEVICE_MAX_TIME_READ) {z.device=0; return 1;}} //--Если тайм-аут превышен, завершение по ошибке  
	   while(Wire.available() < z.qntData);  
	//-----ЧТЕНИЕ РЕГИСТРОВ
	if(z.rdA) z.inA=Wire.read(); else z.inA=z.agoA;
	if(z.rdB) z.inB=Wire.read(); else z.inB=z.agoB;
return 1;}				//--Нормальный выход	
*/
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++	
//-------------------------------------------------------------------------------------------------
//                  ПОИСК УСТРОЙСТВА НА ШИНЕ i2c 
// device: =0 нет устройства; =1 после поиска устройство не обнаружено; =2 устройство обнаружено и доступно к работе.
//-------------------------------------------------------------------------------------------------
/*
void RT_HW_MCP23017:: find (uint8_t &step, uint8_t &addr, uint8_t &device, uint8_t &cnt, uint32_t &timeStart) {//uint8_t code;
switch(step){default: step=0;  
  case 0: switch(device){default: step=0; case 0: step=5; break; case 1: step=1; break;  case 2: break;}	 break; //--Определение точки входа.	 
  //--Поиск, если ранее устройство не было обнаружено--------------------------------------------------------------------------------       
  case 1: timeStart=millis(); step=2;
  case 2: if(getPastTime(timeStart, millis()) < DEVICE_I2C_DELAY_RESTART_LONG)  break;  //--Длинная пауза 
          Wire.beginTransmission(addr); if(Wire.endTransmission()) {device=1;   step=0;} else {timeStart=millis(); step=3;} break;  //--Переход на нормальный поиск или на начало
  case 3: if(getPastTime(timeStart, millis()) < DEVICE_I2C_DELAY_RESTART_SHORT)	break;  //--Короткая пауза перед контрольным поиском 
  case 4: Wire.beginTransmission(addr); if(Wire.endTransmission()) {device=1;   step=0;} else {device=2; step=0; }          break;  //--Завершение поиска     
  //--Первичный поиск (ранее поиск устройства не осуществлялся)----------------------------------------------------------------------                   
  case 5: Wire.beginTransmission(addr); if(Wire.endTransmission()) {            step=7;} else {timeStart=millis(); step=6;} break;  //--Первичный поиск
  case 6: if(getPastTime(timeStart, millis()) < DEVICE_I2C_DELAY_RESTART_SHORT)	break;  //--Короткая пауза перед контрольным поиском
          Wire.beginTransmission(addr); if(Wire.endTransmission()) {            step=7;} else {device=2; step=0;}           break;  //--Продолжение или успешное завершение поиска
  //--Обработка ошибок при первичном поиске-----------------------------------------------------------------------------------------
  case 7: cnt++; if(cnt >=DEVICE_I2C_QNT_RESTART){device=1; step=0; break;} timeStart=millis();step=8;	//--Завершение или подготовка  к повторному поиску
  case 8: if(getPastTime(timeStart,millis()) < DEVICE_I2C_DELAY_RESTART_MIDDLE) break; step=5; break;}}	//--Пауза для повторного цикла   
 */ 
  

  
  
//---------------------------------------------------------------------------------------
//         ВЫЧИСЛЕНИЕ РАЗНИЦЫ МЕЖДУ ДВУМЯ ВРЕМЕННЫМИ ТОЧКАМИ
//---------------------------------------------------------------------------------------
//uint32_t RT_HW_MCP23017:: getPastTime (uint32_t &t1, uint32_t t2){if (t2>=t1) {return (t2-t1);} else {return (0xFFFFFFFF-t1+t2);}};  
//---------------------------------------------------------------------------------------
//            ПЕРИОДИЧЕСКОЕ (ms) ВЫПОЛНЕНИЕ ДЕЙСТВИЙ: рекомендуется инсталлировать t1=millis();
//---------------------------------------------------------------------------------------
 //bool RT_HW_MCP23017:: event_ms  (uint32_t &t1,uint32_t val){if(getPastTime (t1,millis())>=val) {t1+=val; return 1;} return 0;};
// bool RT_HW_MCP23017:: event_mcs (uint32_t &t1,uint32_t val){if(getPastTime (t1,micros())>=val) {t1+=val; return 1;} return 0;}; 
//---------------------------------------------------------------------------------------
//       УСТАНОВКА ПАРАМЕТРА id.run 
//---------------------------------------------------------------------------------------
//void RT_HW_MCP23017::makeRun_device (RT_HW_MCP23017_DEVICE &z,bool en) {
//if(en){if(!z.en){z.en=1;z.run=3;z.count=0;} else {if(z.event){z.count++; if(z.count>=z.period){z.count=0; if(!z.run) z.run=1;}}}} else {z.en=0;}} ;
  


